--
-- 8640
-- Base class for JD8640
--
-- @author  Acid_Burn84
-- @date  08/04/07

Vehicle4 = {};

function Vehicle4:new(configFile, positionX, offsetY, positionZ, yRot, customMt)

    if Vehicle4_mt == nil then
       Vehicle4_mt = Class(Vehicle4);
    end;

    local instance = {};
    if customMt ~= nil then
        setmetatable(instance, customMt);
    else
        setmetatable(instance, Vehicle4_mt);
    end;
    
    instance.configFileName = configFile;

    local xmlFile = loadXMLFile("TempConfig", configFile);

    local rootNode = loadI3DFile(getXMLString(xmlFile, "vehicle.filename"));
    instance.rootNode = getChildAt(rootNode, 0)
    link(getRootNode(), instance.rootNode);
    delete(rootNode);

    local terrainHeight = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, positionX, 300, positionZ);

    setTranslation(instance.rootNode, positionX, terrainHeight+offsetY, positionZ);
    setRotation(instance.rootNode, 0, yRot, 0);

    instance.numWheels = getXMLInt(xmlFile, "vehicle.numWheels");
    if instance.numWheels == nil then
        instance.numWheels = 0;
    end;

    instance.maxRotTime = 0;
    instance.minRotTime = 0;
    instance.autoRotateBackSpeed = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.wheels#autoRotateBackSpeed"), 1.0);
    instance.wheels = {};
    for i=1, instance.numWheels do
        local wheelnamei = string.format("vehicle.wheels.wheel" .. "%d", i);
        instance.wheels[i] = {};
        instance.wheels[i].rotSpeed = Utils.degToRad(getXMLFloat(xmlFile, wheelnamei .. "#rotSpeed"));
        instance.wheels[i].rotMax = Utils.degToRad(getXMLFloat(xmlFile, wheelnamei .. "#rotMax"));
        instance.wheels[i].rotMin = Utils.degToRad(getXMLFloat(xmlFile, wheelnamei .. "#rotMin"));
        instance.wheels[i].driveMode = Utils.getNoNil(getXMLInt(xmlFile, wheelnamei .. "#driveMode"), 0);
        instance.wheels[i].repr = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, wheelnamei .. "#repr"));
        instance.wheels[i].driveNode = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, wheelnamei .. "#driveNode"));
        if instance.wheels[i].driveNode == nil then
            instance.wheels[i].driveNode = instance.wheels[i].repr;
        end;
        local radius = getXMLFloat(xmlFile, wheelnamei .. "#radius");
        radius = Utils.getNoNil(radius, 1);
        _=[[local positionX = getXMLFloat(xmlFile, wheelnamei .. "#positionX");
        positionX = Utils.getNoNil(positionX, 0);
        local positionY = getXMLFloat(xmlFile, wheelnamei .. "#positionY");
        positionY = Utils.getNoNil(positionY, 0);
        local positionZ = getXMLFloat(xmlFile, wheelnamei .. "#positionZ");
        positionZ = Utils.getNoNil(positionZ, 0);]]
        local positionX, positionY, positionZ = getTranslation(instance.wheels[i].repr);
        instance.wheels[i].deltaY = Utils.getNoNil(getXMLFloat(xmlFile, wheelnamei .. "#deltaY"), 0.0);
        positionY = positionY+instance.wheels[i].deltaY;
        local suspTravel = getXMLFloat(xmlFile, wheelnamei .. "#suspTravel");
        suspTravel = Utils.getNoNil(suspTravel, 0);
        local spring = getXMLFloat(xmlFile, wheelnamei .. "#spring");
        spring = Utils.getNoNil(spring, 0);
        local damper = getXMLFloat(xmlFile, wheelnamei .. "#damper");
        damper = Utils.getNoNil(damper, 0);
        local mass = getXMLFloat(xmlFile, wheelnamei .. "#mass");
        mass = Utils.getNoNil(mass, 0.01);
        --local nodeIndexStr = getXMLString(xmlFile, wheelnamei .. "#node");
        _=[[if nodeIndexStr == nil or nodeIndexStr == "" then
            instance.wheels[i].node = instance.rootNode
        else
            instance.wheels[i].node = Utils.indexToObject(instance.rootNode, nodeIndexStr);
        end;]]
        instance.wheels[i].hasGroundContact = false;
        instance.wheels[i].axleSpeed = 0;
        instance.wheels[i].hasHandbrake = true;
        instance.wheels[i].node = getParent(instance.wheels[i].repr);
        instance.wheels[i].wheelShape = createWheelShape(instance.wheels[i].node, positionX, positionY, positionZ, radius, suspTravel, spring, damper, mass);

        local maxRotTime = instance.wheels[i].rotMax/instance.wheels[i].rotSpeed;
        local minRotTime = instance.wheels[i].rotMin/instance.wheels[i].rotSpeed;
        if minRotTime > maxRotTime then
            local temp = minRotTime;
            minRotTime = maxRotTime;
            maxRotTime = temp;
        end;
        if maxRotTime > instance.maxRotTime then
            instance.maxRotTime = maxRotTime;
        end;
        if minRotTime < instance.minRotTime then
            instance.minRotTime = minRotTime;
        end;
    end;
    
    instance.steering = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.steering#index"));
    if instance.steering ~= nil then
        instance.steeringSpeed = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.steering#rotationSpeed"), 0);
    end;

    instance.fan = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.fan#index"));
    instance.fanRotation = 0;

    instance.incrementRotation = true;
    instance.exhaustCover = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.exhaustCover#index"));
    if instance.exhaustCover ~= nil then
        instance.exhaustRotationSpeed = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.exhaustCover#rotationSpeed"), 0);
	  instance.exhaustRotationMax = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.exhaustCover#rotationMax"), 0);
	  instance.exhaustRotationMin = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.exhaustCover#rotationMin"), 0);
	  instance.currentRotation = instance.exhaustRotationMin;
    end;

    local motorMinRpm = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#minRpm"), 1000);
    local motorMaxRpmStr = getXMLString(xmlFile, "vehicle.motor#maxRpm");
    local motorMaxRpm1, motorMaxRpm2, motorMaxRpm3 = Utils.getVectorFromString(motorMaxRpmStr);
    motorMaxRpm1 = Utils.getNoNil(motorMaxRpm1, 800);
    motorMaxRpm2 = Utils.getNoNil(motorMaxRpm2, 1000);
    motorMaxRpm3 = Utils.getNoNil(motorMaxRpm3, 1800);
    local motorMaxRpm = {motorMaxRpm1, motorMaxRpm2, motorMaxRpm3};
    local motorTorque = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#torque"), 15);
    local brakeForce = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#brakeForce"), 10);
    local forwardGearRatio = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#forwardGearRatio"), 1);
    local backwardGearRatio = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#backwardGearRatio"), 1.5);
    local differentialRatio = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#differentialRatio"), 1);
    local rpmFadeOutRange = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#rpmFadeOutRange"), 20);
    local torqueCurve = AnimCurve:new(linearInterpolator1);
    local torqueI = 0;
    while true do
        local key = string.format("vehicle.motor.torque(%d)", torqueI);
        local rpm = getXMLFloat(xmlFile, key.."#rpm");
        local torque = getXMLFloat(xmlFile, key.."#torque");
        if torque == nil or rpm == nil then
            break;
        end;
        torqueCurve:addKeyframe({v=torque, time = rpm});
        torqueI = torqueI +1;
    end;
    _=[[local torqueCurve = AnimCurve:new(linearInterpolator1);
    torqueCurve:addKeyframe({v=2.5, time = 500});
    torqueCurve:addKeyframe({v=4, time = 900});
    torqueCurve:addKeyframe({v=5, time = 1100});
    torqueCurve:addKeyframe({v=5, time = 1400});
    torqueCurve:addKeyframe({v=3, time = 1800});]]
    instance.motor = VehicleMotor:new(motorMinRpm, motorMaxRpm, torqueCurve, brakeForce, forwardGearRatio, backwardGearRatio, differentialRatio, rpmFadeOutRange);
    instance.lastWheelRpm = 0;
    instance.movingDirection = 0;

    instance.fuelCapacity = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fuelCapacity"), 500);
    instance.fuelUsage = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fuelUsage"), 0.01);

    instance.downForce = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.downForce"), 2);

    instance.numCameras = getXMLInt(xmlFile, "vehicle.cameras#count");
    if instance.numCameras == nil or instance.numCameras == 0 then
        instance.numCameras = 0;
        print("Error: no cameras in xml file: ", configFile);
    end;
    instance.cameras = {};
    for i=1, instance.numCameras do
        local cameranamei = string.format("vehicle.cameras.camera%d", i);
        local camIndexStr = getXMLString(xmlFile, cameranamei .. "#index");
        local cameraNode = Utils.indexToObject(instance.rootNode, camIndexStr);
        local rotatable = getXMLBool(xmlFile, cameranamei .. "#rotatable");
        local limit = getXMLBool(xmlFile, cameranamei .. "#limit");
        local rotMinX = getXMLFloat(xmlFile, cameranamei .. "#rotMinX");
        local rotMaxX = getXMLFloat(xmlFile, cameranamei .. "#rotMaxX");
        local transMin = getXMLFloat(xmlFile, cameranamei .. "#transMin");
        local transMax = getXMLFloat(xmlFile, cameranamei .. "#transMax");
        local rotateNode = "";
        if rotatable then
            rotateNode = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, cameranamei .. "#rotateNode"));
        end;
        --instance.cameras[i] = getXMLInt(xmlFile, cameranamei .. "#index");
        instance.cameras[i] = VehicleCamera:new(cameraNode, rotatable, rotateNode, limit, rotMinX, rotMaxX, transMin, transMax);
    end;

    instance.exitPoint = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.exitPoint#index"));

    instance.numLights = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.lights#count"), 0);
    instance.lights = {};
    for i=1, instance.numLights do
        local lightnamei = string.format("vehicle.lights.light" .. "%d", i);
        instance.lights[i] = Utils.indexToObject(instance.rootNode, getXMLInt(xmlFile, lightnamei .. "#index"));
        setVisibility(instance.lights[i], false);
    end;

    local numCuttingAreas = getXMLInt(xmlFile, "vehicle.cuttingAreas#count");
    if numCuttingAreas == nil then
        numCuttingAreas = 0;
    end;
    instance.cuttingAreas = {}
    for i=1, numCuttingAreas do
        instance.cuttingAreas[i] = {};
        local areanamei = string.format("vehicle.cuttingAreas.cuttingArea" .. "%d", i);
        local x,y,z = getTranslation(getChildAt(instance.rootNode, getXMLInt(xmlFile, areanamei .. "#startIndex")));
        local widthX, widthY, widthZ = getTranslation(getChildAt(instance.rootNode, getXMLInt(xmlFile, areanamei .. "#widthIndex")));
        local heightX, heightY, heightZ = getTranslation(getChildAt(instance.rootNode, getXMLInt(xmlFile, areanamei .. "#heightIndex")));
        instance.cuttingAreas[i].startX = x;
        instance.cuttingAreas[i].startZ = z;
        instance.cuttingAreas[i].width = widthX-x;
        instance.cuttingAreas[i].height = heightZ-z;
    end;


    local motorStartSound = getXMLString(xmlFile, "vehicle.motorStartSound#file");
    if motorStartSound ~= nil and motorStartSound ~= "" then
        instance.motorStartSound = createSample("motorStartSound");
        loadSample(instance.motorStartSound, motorStartSound, false);
        instance.motorStartSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorStartSound#pitchOffset"), 0);
    end;

    local attachSound = getXMLString(xmlFile, "vehicle.attachSound#file");
    if attachSound ~= nil and attachSound ~= "" then
        instance.attachSound = createSample("attachSound");
        loadSample(instance.attachSound, attachSound, false);
        instance.attachSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.attachSound#pitchOffset"), 0);
    end;

    local motorSound = getXMLString(xmlFile, "vehicle.motorSound#file");
    if motorSound ~= nil and motorSound ~= "" then
        instance.motorSound = createSample("motorSound");
        loadSample(instance.motorSound, motorSound, false);
        instance.motorSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorSound#pitchOffset"), 0);
        instance.motorSoundPitchScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorSound#pitchScale"), 0.05);
        instance.motorSoundPitchMax = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorSound#pitchMax"), 2.0);
    end;

    local reverseDriveSound = getXMLString(xmlFile, "vehicle.reverseDriveSound#file");
    if reverseDriveSound ~= nil and reverseDriveSound ~= "" then
        instance.reverseDriveSound = createSample("reverseDriveSound");
        instance.reverseDriveSoundEnabled = false;
        loadSample(instance.reverseDriveSound, reverseDriveSound, false);
    end;

    _=[[local weight = getXMLFloat(xmlFile, "vehicle.weight");
    if weight ~= nil then
        print("set mass: ", weight);
        setMass(instance.rootNode, weight/1000.0);
    end;]]
    -- this isnt working correctly yet

    --print("mass: ", getMass(instance.rootNode));

    local centerOfMassX = getXMLFloat(xmlFile, "vehicle.centerOfMass#x");
    local centerOfMassY = getXMLFloat(xmlFile, "vehicle.centerOfMass#y");
    local centerOfMassZ = getXMLFloat(xmlFile, "vehicle.centerOfMass#z");
    if centerOfMassX ~= nil and centerOfMassY ~= nil and centerOfMassZ ~= nil then
        setCenterOfMass(instance.rootNode, centerOfMassX, centerOfMassY, centerOfMassZ);
    end

    instance.trailerAttacherJoint = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.attacherJoint#index"));
    instance.trailerAttacherJointIndex = 0;
    instance.attachedTrailer = nil;

    instance.attacherJoints = {};
    local i=0;
    while true do
        local baseName = string.format("vehicle.attacherJoints.attacherJoint(%d)", i);
        local index = getXMLString(xmlFile, baseName.. "#index");
        if index == nil then
            break;
        end;
        local object = Utils.indexToObject(instance.rootNode, index);
        if object ~= nil then
            local entry = {};
            entry.jointTransform = object;
            local x, y, z;
            local rotationNode = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, baseName.. "#rotationNode"));
            if rotationNode ~= nil then
                entry.rotationNode = rotationNode;
                x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, baseName.."#maxRot"));
                entry.maxRot = {};
                entry.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
                entry.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
                entry.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

                x, y, z = getRotation(rotationNode);
                entry.minRot = {x,y,z};
            end;
            local rotationNode2 = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, baseName.. "#rotationNode2"));
            if rotationNode2 ~= nil then
                entry.rotationNode2 = rotationNode2;
                x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, baseName.."#maxRot2"));
                entry.maxRot2 = {};
                entry.maxRot2[1] = Utils.degToRad(Utils.getNoNil(x, 0));
                entry.maxRot2[2] = Utils.degToRad(Utils.getNoNil(y, 0));
                entry.maxRot2[3] = Utils.degToRad(Utils.getNoNil(z, 0));

                x, y, z = getRotation(rotationNode2);
                entry.minRot2 = {x,y,z};
            end;


            x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, baseName.."#maxRotLimit"));
            entry.maxRotLimit = {};
            entry.maxRotLimit[1] = Utils.degToRad(Utils.getNoNil(math.abs(x), 0));
            entry.maxRotLimit[2] = Utils.degToRad(Utils.getNoNil(math.abs(y), 0));
            entry.maxRotLimit[3] = Utils.degToRad(Utils.getNoNil(math.abs(z), 20));

            x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, baseName.."#maxTransLimit"));
            entry.maxTransLimit = {};
            entry.maxTransLimit[1] = Utils.getNoNil(math.abs(x), 0);
            entry.maxTransLimit[2] = Utils.getNoNil(math.abs(y), 1);
            entry.maxTransLimit[3] = Utils.getNoNil(math.abs(z), 0);

            entry.moveTime = Utils.getNoNil(getXMLFloat(xmlFile, baseName.."#moveTime"), 0.5)*1000;

            local rotationNode = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, baseName.. ".topArm#rotationNode"));
            local translationNode = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, baseName.. ".topArm#translationNode"));
            local referenceNode = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, baseName.. ".topArm#referenceNode"));
            if rotationNode ~= nil then
                local topArm = {};
                topArm.rotationNode = rotationNode;
                topArm.rotX, topArm.rotY, topArm.rotZ = getRotation(rotationNode);
                if translationNode ~= nil and referenceNode ~= nil then
                    topArm.translationNode = translationNode;
                    local ax, ay, az = getWorldTranslation(referenceNode);
                    local bx, by, bz = getWorldTranslation(translationNode);
                    topArm.referenceDistance = Utils.vector3Length(ax-bx, ay-by, az-bz);
                end;
                topArm.zScale = Utils.sign(Utils.getNoNil(getXMLFloat(xmlFile, baseName.. ".topArm#zScale"), 1));
                entry.topArm = topArm;
            end;
            local rotationNode = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, baseName.. ".bottomArm#rotationNode"));
            local translationNode = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, baseName.. ".bottomArm#translationNode"));
            local referenceNode = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, baseName.. ".bottomArm#referenceNode"));
            if rotationNode ~= nil then
                local bottomArm = {};
                bottomArm.rotationNode = rotationNode;
                bottomArm.rotX, bottomArm.rotY, bottomArm.rotZ = getRotation(rotationNode);
                if translationNode ~= nil and referenceNode ~= nil then
                    bottomArm.translationNode = translationNode;
                    local ax, ay, az = getWorldTranslation(referenceNode);
                    local bx, by, bz = getWorldTranslation(translationNode);
                    bottomArm.referenceDistance = Utils.vector3Length(ax-bx, ay-by, az-bz);
                end;
                bottomArm.zScale = Utils.sign(Utils.getNoNil(getXMLFloat(xmlFile, baseName.. ".bottomArm#zScale"), 1));
                entry.bottomArm = bottomArm;
            end;
            entry.jointIndex = 0;
            table.insert(instance.attacherJoints, entry);
        end;
        i = i+1;
    end;
    --instance.attachedAttachableIndex = 0;
    instance.attachedImplements = {};
    instance.selectedImplement = 0;

    instance.tipCamera = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.tipCamera#index"));
    instance.exhaustNode = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.exhaustNode#index"));


    instance.exhaustParticleSystems = {};
    local exhaustParticleSystemCount = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.exhaustParticleSystems#count"), 0);
    for i=1, exhaustParticleSystemCount do
        local namei = string.format("vehicle.exhaustParticleSystems.exhaustParticleSystem" .. "%d", i);
        Utils.loadParticleSystem(xmlFile, instance.exhaustParticleSystems, namei, instance.exhaustNode, false)
    end;

	local rotationPartNodeAttacherPart = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.rotationPartAttacherPart#index"));
    if rotationPartNodeAttacherPart ~= nil then
        instance.rotationPartAttacherPart = {};
        instance.rotationPartAttacherPart.node = rotationPartNodeAttacherPart;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartAttacherPart#minRot"));
        instance.rotationPartAttacherPart.minRot = {};
        instance.rotationPartAttacherPart.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartAttacherPart.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartAttacherPart.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartAttacherPart#maxRot"));
        instance.rotationPartAttacherPart.maxRot = {};
        instance.rotationPartAttacherPart.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartAttacherPart.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartAttacherPart.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.rotationPartAttacherPart.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartAttacherPart#rotTime"), 2)*1000;
        instance.rotationPartAttacherPart.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartAttacherPart#touchRotLimit"), 10));
    end;
	
	local rotationPartNodeWischer = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.rotationPartWischer#index"));
    if rotationPartNodeWischer ~= nil then
        instance.rotationPartWischer = {};
        instance.rotationPartWischer.node = rotationPartNodeWischer;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartWischer#minRot"));
        instance.rotationPartWischer.minRot = {};
        instance.rotationPartWischer.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartWischer.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartWischer.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartWischer#maxRot"));
        instance.rotationPartWischer.maxRot = {};
        instance.rotationPartWischer.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartWischer.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartWischer.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.rotationPartWischer.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartWischer#rotTime"), 2)*1000;
        instance.rotationPartWischer.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartWischer#touchRotLimit"), 10));
    end;
	
	local rotationPartNodeDrehzahl1 = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.rotationPartDrehzahl1#index"));
    if rotationPartNodeDrehzahl1 ~= nil then
        instance.rotationPartDrehzahl1 = {};
        instance.rotationPartDrehzahl1.node = rotationPartNodeDrehzahl1;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartDrehzahl1#minRot"));
        instance.rotationPartDrehzahl1.minRot = {};
        instance.rotationPartDrehzahl1.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartDrehzahl1.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartDrehzahl1.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartDrehzahl1#maxRot"));
        instance.rotationPartDrehzahl1.maxRot = {};
        instance.rotationPartDrehzahl1.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartDrehzahl1.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartDrehzahl1.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.rotationPartDrehzahl1.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartDrehzahl1#rotTime"), 2)*1000;
        instance.rotationPartDrehzahl1.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartDrehzahl1#touchRotLimit"), 10));
    end;
	
	local rotationPartNodeKmh1 = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.rotationPartKmh1#index"));
    if rotationPartNodeKmh1 ~= nil then
        instance.rotationPartKmh1 = {};
        instance.rotationPartKmh1.node = rotationPartNodeKmh1;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartKmh1#minRot"));
        instance.rotationPartKmh1.minRot = {};
        instance.rotationPartKmh1.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartKmh1.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartKmh1.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartKmh1#maxRot"));
        instance.rotationPartKmh1.maxRot = {};
        instance.rotationPartKmh1.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartKmh1.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartKmh1.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.rotationPartKmh1.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartKmh1#rotTime"), 2)*1000;
        instance.rotationPartKmh1.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartKmh1#touchRotLimit"), 10));
    end;
	
	local rotationPartNodeDrehzahl2 = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.rotationPartDrehzahl2#index"));
    if rotationPartNodeDrehzahl2 ~= nil then
        instance.rotationPartDrehzahl2 = {};
        instance.rotationPartDrehzahl2.node = rotationPartNodeDrehzahl2;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartDrehzahl2#minRot"));
        instance.rotationPartDrehzahl2.minRot = {};
        instance.rotationPartDrehzahl2.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartDrehzahl2.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartDrehzahl2.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartDrehzahl2#maxRot"));
        instance.rotationPartDrehzahl2.maxRot = {};
        instance.rotationPartDrehzahl2.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartDrehzahl2.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartDrehzahl2.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.rotationPartDrehzahl2.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartDrehzahl2#rotTime"), 2)*1000;
        instance.rotationPartDrehzahl2.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartDrehzahl2#touchRotLimit"), 10));
    end;
	
	local rotationPartNodeKmh2 = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.rotationPartKmh2#index"));
    if rotationPartNodeKmh2 ~= nil then
        instance.rotationPartKmh2 = {};
        instance.rotationPartKmh2.node = rotationPartNodeKmh2;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartKmh2#minRot"));
        instance.rotationPartKmh2.minRot = {};
        instance.rotationPartKmh2.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartKmh2.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartKmh2.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartKmh2#maxRot"));
        instance.rotationPartKmh2.maxRot = {};
        instance.rotationPartKmh2.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartKmh2.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartKmh2.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.rotationPartKmh2.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartKmh2#rotTime"), 2)*1000;
        instance.rotationPartKmh2.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartKmh2#touchRotLimit"), 10));
    end;
	
	local rotationPartNodeBlinkerLinks = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.rotationPartBlinkerLinks#index"));
    if rotationPartNodeBlinkerLinks ~= nil then
        instance.rotationPartBlinkerLinks = {};
        instance.rotationPartBlinkerLinks.node = rotationPartNodeBlinkerLinks;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartBlinkerLinks#minRot"));
        instance.rotationPartBlinkerLinks.minRot = {};
        instance.rotationPartBlinkerLinks.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartBlinkerLinks.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartBlinkerLinks.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartBlinkerLinks#maxRot"));
        instance.rotationPartBlinkerLinks.maxRot = {};
        instance.rotationPartBlinkerLinks.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartBlinkerLinks.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartBlinkerLinks.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.rotationPartBlinkerLinks.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartBlinkerLinks#rotTime"), 2)*1000;
        instance.rotationPartBlinkerLinks.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartBlinkerLinks#touchRotLimit"), 10));
    end;
	
	local rotationPartNodeBlinkerRechts = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.rotationPartBlinkerRechts#index"));
    if rotationPartNodeBlinkerRechts ~= nil then
        instance.rotationPartBlinkerRechts = {};
        instance.rotationPartBlinkerRechts.node = rotationPartNodeBlinkerRechts;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartBlinkerRechts#minRot"));
        instance.rotationPartBlinkerRechts.minRot = {};
        instance.rotationPartBlinkerRechts.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartBlinkerRechts.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartBlinkerRechts.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartBlinkerRechts#maxRot"));
        instance.rotationPartBlinkerRechts.maxRot = {};
        instance.rotationPartBlinkerRechts.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartBlinkerRechts.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartBlinkerRechts.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.rotationPartBlinkerRechts.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartBlinkerRechts#rotTime"), 2)*1000;
        instance.rotationPartBlinkerRechts.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartBlinkerRechts#touchRotLimit"), 10));
    end;
	
	local rotationPartNodeWischerblatt = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.rotationPartWischerblatt#index"));
    if rotationPartNodeWischerblatt ~= nil then
        instance.rotationPartWischerblatt = {};
        instance.rotationPartWischerblatt.node = rotationPartNodeWischerblatt;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartWischerblatt#minRot"));
        instance.rotationPartWischerblatt.minRot = {};
        instance.rotationPartWischerblatt.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartWischerblatt.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartWischerblatt.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartWischerblatt#maxRot"));
        instance.rotationPartWischerblatt.maxRot = {};
        instance.rotationPartWischerblatt.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartWischerblatt.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartWischerblatt.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.rotationPartWischerblatt.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartWischerblatt#rotTime"), 2)*1000;
        instance.rotationPartWischerblatt.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartWischerblatt#touchRotLimit"), 10));
    end;
	
	local rotationPartNodeTanknadel = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.rotationPartTanknadel#index"));
    if rotationPartNodeTanknadel ~= nil then
        instance.rotationPartTanknadel = {};
        instance.rotationPartTanknadel.node = rotationPartNodeTanknadel;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartTanknadel#minRot"));
        instance.rotationPartTanknadel.minRot = {};
        instance.rotationPartTanknadel.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartTanknadel.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartTanknadel.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartTanknadel#maxRot"));
        instance.rotationPartTanknadel.maxRot = {};
        instance.rotationPartTanknadel.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartTanknadel.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartTanknadel.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.rotationPartTanknadel.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartTanknadel#rotTime"), 2)*1000;
        instance.rotationPartTanknadel.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartTanknadel#touchRotLimit"), 10));
    end;
	
	local rotationPartNodeTanknadel2 = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.rotationPartTanknadel2#index"));
    if rotationPartNodeTanknadel2 ~= nil then
        instance.rotationPartTanknadel2 = {};
        instance.rotationPartTanknadel2.node = rotationPartNodeTanknadel2;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartTanknadel2#minRot"));
        instance.rotationPartTanknadel2.minRot = {};
        instance.rotationPartTanknadel2.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartTanknadel2.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartTanknadel2.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartTanknadel2#maxRot"));
        instance.rotationPartTanknadel2.maxRot = {};
        instance.rotationPartTanknadel2.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartTanknadel2.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartTanknadel2.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.rotationPartTanknadel2.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartTanknadel2#rotTime"), 2)*1000;
        instance.rotationPartTanknadel2.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartTanknadel2#touchRotLimit"), 10));
    end;
	
	local rotationPartNodePedal = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.rotationPartPedal#index"));
    if rotationPartNodePedal ~= nil then
        instance.rotationPartPedal = {};
        instance.rotationPartPedal.node = rotationPartNodePedal;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartPedal#minRot"));
        instance.rotationPartPedal.minRot = {};
        instance.rotationPartPedal.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartPedal.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartPedal.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartPedal#maxRot"));
        instance.rotationPartPedal.maxRot = {};
        instance.rotationPartPedal.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartPedal.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartPedal.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.rotationPartPedal.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartPedal#rotTime"), 2)*1000;
        instance.rotationPartPedal.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartPedal#touchRotLimit"), 10));
    end;
	
	local rotationPartNodeBrake = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.rotationPartBrake#index"));
    if rotationPartNodeBrake ~= nil then
        instance.rotationPartBrake = {};
        instance.rotationPartBrake.node = rotationPartNodeBrake;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartBrake#minRot"));
        instance.rotationPartBrake.minRot = {};
        instance.rotationPartBrake.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartBrake.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartBrake.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartBrake#maxRot"));
        instance.rotationPartBrake.maxRot = {};
        instance.rotationPartBrake.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartBrake.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartBrake.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.rotationPartBrake.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartBrake#rotTime"), 2)*1000;
        instance.rotationPartBrake.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartBrake#touchRotLimit"), 10));
    end;
	
	local rotationPartNodeClutch = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.rotationPartClutch#index"));
    if rotationPartNodeClutch ~= nil then
        instance.rotationPartClutch = {};
        instance.rotationPartClutch.node = rotationPartNodeClutch;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartClutch#minRot"));
        instance.rotationPartClutch.minRot = {};
        instance.rotationPartClutch.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartClutch.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartClutch.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartClutch#maxRot"));
        instance.rotationPartClutch.maxRot = {};
        instance.rotationPartClutch.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartClutch.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartClutch.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.rotationPartClutch.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartClutch#rotTime"), 2)*1000;
        instance.rotationPartClutch.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartClutch#touchRotLimit"), 10));
    end;

	
	local rotationPartNodeDruck = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.rotationPartDruck#index"));
    if rotationPartNodeDruck ~= nil then
        instance.rotationPartDruck = {};
        instance.rotationPartDruck.node = rotationPartNodeDruck;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartDruck#minRot"));
        instance.rotationPartDruck.minRot = {};
        instance.rotationPartDruck.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartDruck.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartDruck.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartDruck#maxRot"));
        instance.rotationPartDruck.maxRot = {};
        instance.rotationPartDruck.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartDruck.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartDruck.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.rotationPartDruck.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartDruck#rotTime"), 2)*1000;
        instance.rotationPartDruck.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartDruck#touchRotLimit"), 10));
    end;
	
	local rotationPartNodeDruckLight = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.rotationPartDruckLight#index"));
    if rotationPartNodeDruckLight ~= nil then
        instance.rotationPartDruckLight = {};
        instance.rotationPartDruckLight.node = rotationPartNodeDruckLight;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartDruckLight#minRot"));
        instance.rotationPartDruckLight.minRot = {};
        instance.rotationPartDruckLight.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartDruckLight.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartDruckLight.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartDruckLight#maxRot"));
        instance.rotationPartDruckLight.maxRot = {};
        instance.rotationPartDruckLight.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartDruckLight.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartDruckLight.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.rotationPartDruckLight.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartDruckLight#rotTime"), 2)*1000;
        instance.rotationPartDruckLight.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartDruckLight#touchRotLimit"), 10));
    end;
	
	local translationPartNode = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.translationPart#index"));
    if translationPartNode ~= nil then
        instance.translationPart = {};
        instance.translationPart.node = translationPartNode;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.translationPart#minTrans"));
        instance.translationPart.minTrans = {};
        instance.translationPart.minTrans[1] = Utils.getNoNil(x, 0);
        instance.translationPart.minTrans[2] = Utils.getNoNil(y, 0);
        instance.translationPart.minTrans[3] = Utils.getNoNil(z, 0);

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.translationPart#maxTrans"));
        instance.translationPart.maxTrans = {};
        instance.translationPart.maxTrans[1] = Utils.getNoNil(x, 0);
        instance.translationPart.maxTrans[2] = Utils.getNoNil(y, 0);
        instance.translationPart.maxTrans[3] = Utils.getNoNil(z, 0);

        instance.translationPart.transTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.translationPart#transTime"), 2)*1000;
        instance.translationPart.touchTransLimit = Utils.getNoNil(getXMLString(xmlFile, "vehicle.translationPart#touchTransLimit"), 10);
    end;
	
	local translationPartNodeAttacher = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.translationPartAttacher#index"));
    if translationPartNodeAttacher ~= nil then
        instance.translationPartAttacher = {};
        instance.translationPartAttacher.node = translationPartNodeAttacher;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.translationPartAttacher#minTrans"));
        instance.translationPartAttacher.minTrans = {};
        instance.translationPartAttacher.minTrans[1] = Utils.getNoNil(x, 0);
        instance.translationPartAttacher.minTrans[2] = Utils.getNoNil(y, 0);
        instance.translationPartAttacher.minTrans[3] = Utils.getNoNil(z, 0);

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.translationPartAttacher#maxTrans"));
        instance.translationPartAttacher.maxTrans = {};
        instance.translationPartAttacher.maxTrans[1] = Utils.getNoNil(x, 0);
        instance.translationPartAttacher.maxTrans[2] = Utils.getNoNil(y, 0);
        instance.translationPartAttacher.maxTrans[3] = Utils.getNoNil(z, 0);

        instance.translationPartAttacher.transTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.translationPartAttacher#transTime"), 2)*1000;
        instance.translationPartAttacher.touchTransLimit = Utils.getNoNil(getXMLString(xmlFile, "vehicle.translationPartAttacher#touchTransLimit"), 10);
    end;


	
	
	instance.exhaustParticleSystems2 = {};
    local psName = "vehicle.exhaustParticleSystemExtra1";
    Utils.loadParticleSystem(xmlFile, instance.exhaustParticleSystems2, psName, instance.exhaustNode, false)
	
	instance.exhaustParticleSystems3 = {};
    local psName = "vehicle.exhaustParticleSystemExtra2";
    Utils.loadParticleSystem(xmlFile, instance.exhaustParticleSystems3, psName, instance.exhaustNode, false)
	
	instance.exhaustParticleSystems4 = {};
    local psName = "vehicle.exhaustParticleSystemExtra3";
    Utils.loadParticleSystem(xmlFile, instance.exhaustParticleSystems4, psName, instance.exhaustNode, false)
	
	instance.exhaustParticleSystems5 = {};
    local psName = "vehicle.exhaustParticleSystemExtra4";
    Utils.loadParticleSystem(xmlFile, instance.exhaustParticleSystems5, psName, instance.exhaustNode, false)
	
	instance.numWorklights = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.worklights#count"), 0);
    instance.worklights = {};
    for i=1, instance.numWorklights do
        local worklightnamei = string.format("vehicle.worklights.worklight" .. "%d", i);
        instance.worklights[i] = Utils.indexToObject(instance.rootNode, getXMLInt(xmlFile, worklightnamei .. "#index"));
        setVisibility(instance.worklights[i], false);
    end;
	
	instance.numFarlights = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.farlights#count"), 0);
    instance.farlights = {};
    for i=1, instance.numFarlights do
        local farlightnamei = string.format("vehicle.farlights.farlight" .. "%d", i);
        instance.farlights[i] = Utils.indexToObject(instance.rootNode, getXMLInt(xmlFile, farlightnamei .. "#index"));
        setVisibility(instance.farlights[i], false);
    end;
	
	instance.numFrontlights = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.frontlights#count"), 0);
    instance.frontlights = {};
    for i=1, instance.numFrontlights do
        local frontlightnamei = string.format("vehicle.frontlights.frontlight" .. "%d", i);
        instance.frontlights[i] = Utils.indexToObject(instance.rootNode, getXMLInt(xmlFile, frontlightnamei .. "#index"));
        setVisibility(instance.frontlights[i], false);
    end;

	instance.lowLightDisplay = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.lowLightDisplay#index"));
	instance.farLightDisplay = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.farLightDisplay#index"));
	instance.workLightDisplay = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.workLightDisplay#index"));
	instance.fuelWarningDisplay = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.fuelWarningDisplay#index"));
	instance.handbrakeDisplay = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.handbrakeDisplay#index"));
	instance.barDisplay = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.barDisplay#index"));
	instance.roundLightDisplay = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.roundLightDisplay#index"));

	instance.rundumleuchtenAnz = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.rundumleuchten#count"),0);
    instance.rundumleuchtenKey = getXMLString(xmlFile, "vehicle.rundumleuchten#key");
    instance.rundumleuchtenAn = false;
    instance.rundumleuchten = {};
    for i=1, instance.rundumleuchtenAnz do
        local objname = string.format("vehicle.rundumleuchten.light" .. "%d",i);
        instance.rundumleuchten[i] = {};
        instance.rundumleuchten[i].rotNode = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, objname .. "#rotNode"));
        instance.rundumleuchten[i].light = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, objname .. "#light"));
        instance.rundumleuchten[i].source = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, objname .. "#lightsource"));
        instance.rundumleuchten[i].speed = Utils.getNoNil(getXMLInt(xmlFile,  objname .. "#rotSpeed"), 1)/1000;
        instance.rundumleuchten[i].emit = Utils.getNoNil(getXMLBool(xmlFile, objname .. "#emitLight"), true);
        if not instance.rundumleuchten[i].emit and instance.rundumleuchten[i].source ~= nil then
           setVisibility(instance.rundumleuchten[i].source, false);
        end;
    end;

	instance.bremzleuchtenAnz = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.bremzleuchten#count"),0);
    instance.bremzleuchtenKey = getXMLString(xmlFile, "vehicle.bremzleuchten#key");
    instance.bremzleuchtenAn = false;
    instance.bremzleuchten = {};
    for i=1, instance.bremzleuchtenAnz do
        local objname = string.format("vehicle.bremzleuchten.light" .. "%d",i);
        instance.bremzleuchten[i] = {};
        instance.bremzleuchten[i].rotNode = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, objname .. "#rotNode"));
        instance.bremzleuchten[i].light = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, objname .. "#light"));
        instance.bremzleuchten[i].source = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, objname .. "#lightsource"));
        instance.bremzleuchten[i].speed = Utils.getNoNil(getXMLInt(xmlFile,  objname .. "#rotSpeed"), 1)/1000;
        instance.bremzleuchten[i].emit = Utils.getNoNil(getXMLBool(xmlFile, objname .. "#emitLight"), true);
        if not instance.bremzleuchten[i].emit and instance.bremzleuchten[i].source ~= nil then
           setVisibility(instance.bremzleuchten[i].source, false);
        end;
    end;

	instance.numrpmdisplays = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.rpmdisplays#count"), 0);
    instance.rpmdisplays = {};
    for i=1, instance.numrpmdisplays do
        local objname = string.format("vehicle.rpmdisplays.rpmdisplay" .. "%d", i);
	  instance.rpmdisplays[i] = {};
        instance.rpmdisplays[i].rotNode = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, objname .. "#rotNode"));
	  instance.rpmdisplays[i].rpmMinValue = Utils.getNoNil(getXMLInt(xmlFile,  objname .. "#rpmMinValue"), 1);
	  instance.rpmdisplays[i].rpmMaxValue = Utils.getNoNil(getXMLInt(xmlFile,  objname .. "#rpmMaxValue"), 1);
        setVisibility(instance.rpmdisplays[i].rotNode, false);
    end;

	instance.numrpmlightdisplays = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.rpmlightdisplays#count"), 0);
    instance.rpmlightdisplays = {};
    for i=1, instance.numrpmlightdisplays do
        local objname = string.format("vehicle.rpmlightdisplays.rpmlightdisplay" .. "%d", i);
	  instance.rpmlightdisplays[i] = {};
        instance.rpmlightdisplays[i].rotNode = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, objname .. "#rotNode"));
	  instance.rpmlightdisplays[i].rpmMinValue = Utils.getNoNil(getXMLInt(xmlFile,  objname .. "#rpmMinValue"), 1);
	  instance.rpmlightdisplays[i].rpmMaxValue = Utils.getNoNil(getXMLInt(xmlFile,  objname .. "#rpmMaxValue"), 1);
        setVisibility(instance.rpmlightdisplays[i].rotNode, false);
    end;


	instance.numTwinWheels = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.twinWheels#count"), 0);
    instance.twinWheels = {};
    for i=1, instance.numTwinWheels do
        local twinWheelnamei = string.format("vehicle.twinWheels.twinWheel" .. "%d", i);
        instance.twinWheels[i] = Utils.indexToObject(instance.rootNode, getXMLInt(xmlFile, twinWheelnamei .. "#index"));
        setVisibility(instance.twinWheels[i], false);
    end;

      instance.numPlowWheels = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.plowWheels#count"), 0);
    instance.plowWheels = {};
    for i=1, instance.numPlowWheels do
        local plowWheelnamei = string.format("vehicle.plowWheels.plowWheel" .. "%d", i);
        instance.plowWheels[i] = Utils.indexToObject(instance.rootNode, getXMLInt(xmlFile, plowWheelnamei .. "#index"));
        setVisibility(instance.plowWheels[i], false);
    end;


    instance.gearShifter = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.gearShifter#index"));
    instance.gearIndicator = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.gearIndicator#index"));

	instance.frontleuchtenAnz = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.frontleuchten#count"),0);
    instance.frontleuchtenKey = getXMLString(xmlFile, "vehicle.frontleuchten#key");
    instance.frontleuchtenAn = false;
    instance.frontleuchten = {};
    for i=1, instance.frontleuchtenAnz do
        local objname = string.format("vehicle.frontleuchten.light" .. "%d",i);
        instance.frontleuchten[i] = {};
        instance.frontleuchten[i].rotNode = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, objname .. "#rotNode"));
        instance.frontleuchten[i].light = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, objname .. "#light"));
        instance.frontleuchten[i].source = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, objname .. "#lightsource"));
        instance.frontleuchten[i].speed = Utils.getNoNil(getXMLInt(xmlFile,  objname .. "#rotSpeed"), 1)/1000;
        instance.frontleuchten[i].emit = Utils.getNoNil(getXMLBool(xmlFile, objname .. "#emitLight"), true);
        if not instance.frontleuchten[i].emit and instance.frontleuchten[i].source ~= nil then
           setVisibility(instance.frontleuchten[i].source, false);
        end;
    end;

	instance.farleuchtenAnz = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.farleuchten#count"),0);
    instance.farleuchtenKey = getXMLString(xmlFile, "vehicle.farleuchten#key");
    instance.farleuchtenAn = false;
    instance.farleuchten = {};
    for i=1, instance.farleuchtenAnz do
        local objname = string.format("vehicle.farleuchten.light" .. "%d",i);
        instance.farleuchten[i] = {};
        instance.farleuchten[i].rotNode = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, objname .. "#rotNode"));
        instance.farleuchten[i].light = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, objname .. "#light"));
        instance.farleuchten[i].source = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, objname .. "#lightsource"));
        instance.farleuchten[i].speed = Utils.getNoNil(getXMLInt(xmlFile,  objname .. "#rotSpeed"), 1)/1000;
        instance.farleuchten[i].emit = Utils.getNoNil(getXMLBool(xmlFile, objname .. "#emitLight"), true);
        if not instance.farleuchten[i].emit and instance.farleuchten[i].source ~= nil then
           setVisibility(instance.farleuchten[i].source, false);
        end;
    end;

      instance.leuchtenAnz = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.leuchten#count"),0);
    instance.leuchtenKey = getXMLString(xmlFile, "vehicle.leuchten#key");
    instance.leuchtenAn = false;
    instance.leuchten = {};
    for i=1, instance.leuchtenAnz do
        local objname = string.format("vehicle.leuchten.light" .. "%d",i);
        instance.leuchten[i] = {};
        instance.leuchten[i].rotNode = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, objname .. "#rotNode"));
        instance.leuchten[i].light = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, objname .. "#light"));
        instance.leuchten[i].source = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, objname .. "#lightsource"));
        instance.leuchten[i].speed = Utils.getNoNil(getXMLInt(xmlFile,  objname .. "#rotSpeed"), 1)/1000;
        instance.leuchten[i].emit = Utils.getNoNil(getXMLBool(xmlFile, objname .. "#emitLight"), true);
        if not instance.leuchten[i].emit and instance.leuchten[i].source ~= nil then
           setVisibility(instance.leuchten[i].source, false);
        end;
    end;


	
	instance.numFlashlightsLeft = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.flashlightsLeft#count"), 0);
    instance.flashlightsLeft = {};
    for i=1, instance.numFlashlightsLeft do
        local flashlightLeftnamei = string.format("vehicle.flashlightsLeft.flashlightLeft" .. "%d", i);
        instance.flashlightsLeft[i] = Utils.indexToObject(instance.rootNode, getXMLInt(xmlFile, flashlightLeftnamei .. "#index"));
        setVisibility(instance.flashlightsLeft[i], false);
    end;
	
	instance.numFlashlightsRight = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.flashlightsRight#count"), 0);
    instance.flashlightsRight = {};
    for i=1, instance.numFlashlightsRight do
        local flashlightRightnamei = string.format("vehicle.flashlightsRight.flashlightRight" .. "%d", i);
        instance.flashlightsRight[i] = Utils.indexToObject(instance.rootNode, getXMLInt(xmlFile, flashlightRightnamei .. "#index"));
        setVisibility(instance.flashlightsRight[i], false);
    end;
	
	instance.numFlashlightsWarner = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.flashlightsWarner#count"), 0);
    instance.flashlightsWarner = {};
    for i=1, instance.numFlashlightsWarner do
        local flashlightWarnernamei = string.format("vehicle.flashlightsWarner.flashlightWarner" .. "%d", i);
        instance.flashlightsWarner[i] = Utils.indexToObject(instance.rootNode, getXMLInt(xmlFile, flashlightWarnernamei .. "#index"));
        setVisibility(instance.flashlightsWarner[i], false);
    end;
	
	instance.numFlashlightsLeft2 = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.flashlightsLeft2#count"), 0);
    instance.flashlightsLeft2 = {};
    for i=1, instance.numFlashlightsLeft2 do
        local flashlightLeft2namei = string.format("vehicle.flashlightsLeft2.flashlightLeft2" .. "%d", i);
        instance.flashlightsLeft2[i] = Utils.indexToObject(instance.rootNode, getXMLInt(xmlFile, flashlightLeft2namei .. "#index"));
        setVisibility(instance.flashlightsLeft2[i], false);
    end;
	
	instance.numFlashlightsRight2 = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.flashlightsRight2#count"), 0);
    instance.flashlightsRight2 = {};
    for i=1, instance.numFlashlightsRight2 do
        local flashlightRight2namei = string.format("vehicle.flashlightsRight2.flashlightRight2" .. "%d", i);
        instance.flashlightsRight2[i] = Utils.indexToObject(instance.rootNode, getXMLInt(xmlFile, flashlightRight2namei .. "#index"));
        setVisibility(instance.flashlightsRight2[i], false);
    end;
	
	instance.numBrakelights = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.brakelights#count"), 0);
    instance.brakelights = {};
    for i=1, instance.numBrakelights do
        local brakelightnamei = string.format("vehicle.brakelights.brakelight" .. "%d", i);
        instance.brakelights[i] = Utils.indexToObject(instance.rootNode, getXMLInt(xmlFile, brakelightnamei .. "#index"));
        setVisibility(instance.brakelights[i], false);
    end;
	
	instance.numFuelWarnlights = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.fuelWarnlights#count"), 0);
    instance.fuelWarnlights = {};
    for i=1, instance.numFuelWarnlights do
        local fuelWarnlightnamei = string.format("vehicle.fuelWarnlights.fuelWarnlight" .. "%d", i);
        instance.fuelWarnlights[i] = Utils.indexToObject(instance.rootNode, getXMLInt(xmlFile, fuelWarnlightnamei .. "#index"));
        setVisibility(instance.fuelWarnlights[i], false);
    end;
	
	instance.numBrakeWarnlights = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.brakeWarnlights#count"), 0);
    instance.brakeWarnlights = {};
    for i=1, instance.numBrakeWarnlights do
        local brakeWarnlightnamei = string.format("vehicle.brakeWarnlights.brakeWarnlight" .. "%d", i);
        instance.brakeWarnlights[i] = Utils.indexToObject(instance.rootNode, getXMLInt(xmlFile, brakeWarnlightnamei .. "#index"));
        setVisibility(instance.brakeWarnlights[i], false);
    end;
	
	instance.numDruckWarnlights = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.druckWarnlights#count"), 0);
    instance.druckWarnlights = {};
    for i=1, instance.numDruckWarnlights do
        local druckWarnlightnamei = string.format("vehicle.druckWarnlights.druckWarnlight" .. "%d", i);
        instance.druckWarnlights[i] = Utils.indexToObject(instance.rootNode, getXMLInt(xmlFile, druckWarnlightnamei .. "#index"));
        setVisibility(instance.druckWarnlights[i], false);
    end;
	
	instance.numDrucklights = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.drucklights#count"), 0);
    instance.drucklights = {};
    for i=1, instance.numDrucklights do
        local drucklightnamei = string.format("vehicle.drucklights.drucklight" .. "%d", i);
        instance.drucklights[i] = Utils.indexToObject(instance.rootNode, getXMLInt(xmlFile, drucklightnamei .. "#index"));
        setVisibility(instance.drucklights[i], false);
    end;
	
	local compressorSound = getXMLString(xmlFile, "vehicle.compressorSound#file");
    if compressorSound ~= nil and compressorSound ~= "" then
        instance.compressorSound = createSample("compressorSound");
        loadSample(instance.compressorSound, compressorSound, false);
    end;
	
	local compressorStopSound = getXMLString(xmlFile, "vehicle.compressorStopSound#file");
    if compressorStopSound ~= nil and compressorStopSound ~= "" then
        instance.compressorStopSound = createSample("compressorStopSound");
        loadSample(instance.compressorStopSound, compressorStopSound, false);
    end;
	
	local brakeSound = getXMLString(xmlFile, "vehicle.brakeSound#file");
    if brakeSound ~= nil and brakeSound ~= "" then
        instance.brakeSound = createSample("brakeSound");
        loadSample(instance.brakeSound, brakeSound, false);
    end;
	
	local brakeReleaseSound = getXMLString(xmlFile, "vehicle.brakeReleaseSound#file");
    if brakeReleaseSound ~= nil and brakeReleaseSound ~= "" then
        instance.brakeReleaseSound = createSample("brakeReleaseSound");
        loadSample(instance.brakeReleaseSound, brakeReleaseSound, false);
    end;
	
	local fuelWarnSound = getXMLString(xmlFile, "vehicle.fuelWarnSound#file");
    if fuelWarnSound ~= nil and fuelWarnSound ~= "" then
        instance.fuelWarnSound = createSample("fuelWarnSound");
        loadSample(instance.fuelWarnSound, fuelWarnSound, false);
    end;
	
	local turnSound = getXMLString(xmlFile, "vehicle.turnSound#file");
    if turnSound ~= nil and turnSound ~= "" then
        instance.turnSound = createSample("turnSound");
        loadSample(instance.turnSound, turnSound, false);
    end;
	
	local turnSound2 = getXMLString(xmlFile, "vehicle.turnSound2#file");
    if turnSound2 ~= nil and turnSound2 ~= "" then
        instance.turnSound2 = createSample("turnSound2");
        loadSample(instance.turnSound2, turnSound2, false);
    end;
	
	local handbrakeSound = getXMLString(xmlFile, "vehicle.handbrakeSound#file");
    if handbrakeSound ~= nil and handbrakeSound ~= "" then
        instance.handbrakeSound = createSample("handbrakeSound");
        loadSample(instance.handbrakeSound, handbrakeSound, false);
    end;
--Frontlader Anfang
	
	local jointDesc = {};

    jointDesc.jointTransform = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.frontLoaderJoint#index"));
    if jointDesc.jointTransform ~= nil then
        jointDesc.rotationNode = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.frontLoaderJoint#rotationNode"));
        if jointDesc.rotationNode ~= nil then
            local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.frontLoaderJoint#maxRot"));
            jointDesc.maxRot = {};
            jointDesc.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
            jointDesc.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
            jointDesc.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

            x, y, z = getRotation(jointDesc.rotationNode);
            jointDesc.minRot = {x,y,z};
        end;
        jointDesc.rotationNode2 = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.frontLoaderJoint#rotationNode2"));
        if jointDesc.rotationNode2 ~= nil then
            local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.frontLoaderJoint#maxRot2"));
            jointDesc.maxRot2 = {};
            jointDesc.maxRot2[1] = Utils.degToRad(Utils.getNoNil(x, 0));
            jointDesc.maxRot2[2] = Utils.degToRad(Utils.getNoNil(y, 0));
            jointDesc.maxRot2[3] = Utils.degToRad(Utils.getNoNil(z, 0));

            x, y, z = getRotation(jointDesc.rotationNode2);
            jointDesc.minRot2 = {x,y,z};
        end;
        jointDesc.moveTime = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.frontLoaderJoint#moveTime"), 0.5)*1000;
        jointDesc.moveTime2 = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.frontLoaderJoint#moveTime2"), 0.5)*1000;

        jointDesc.moveDirection = 0; -- 1 = rotate up, -1 = rotate down
        jointDesc.moveDirection2 = 0;

        jointDesc.jointIndex = 0;
    end;

    instance.frontLoaderJointDesc = jointDesc;
    instance.frontLoaderAttachedObject = nil;
	
	local rotationPartNodeZylinderHub = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.rotationPartZylinderHub#index"));
    if rotationPartNodeZylinderHub ~= nil then
        instance.rotationPartZylinderHub = {};
        instance.rotationPartZylinderHub.node = rotationPartNodeZylinderHub;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartZylinderHub#minRot"));
        instance.rotationPartZylinderHub.minRot = {};
        instance.rotationPartZylinderHub.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartZylinderHub.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartZylinderHub.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartZylinderHub#maxRot"));
        instance.rotationPartZylinderHub.maxRot = {};
        instance.rotationPartZylinderHub.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartZylinderHub.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartZylinderHub.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.rotationPartZylinderHub.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartZylinderHub#rotTime"), 2)*1000;
        instance.rotationPartZylinderHub.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartZylinderHub#touchRotLimit"), 10));
    end;
	
	local rotationPartNodeZylinderKipp = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.rotationPartZylinderKipp#index"));
    if rotationPartNodeZylinderKipp ~= nil then
        instance.rotationPartZylinderKipp = {};
        instance.rotationPartZylinderKipp.node = rotationPartNodeZylinderKipp;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartZylinderKipp#minRot"));
        instance.rotationPartZylinderKipp.minRot = {};
        instance.rotationPartZylinderKipp.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartZylinderKipp.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartZylinderKipp.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartZylinderKipp#maxRot"));
        instance.rotationPartZylinderKipp.maxRot = {};
        instance.rotationPartZylinderKipp.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartZylinderKipp.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartZylinderKipp.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.rotationPartZylinderKipp.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartZylinderKipp#rotTime"), 2)*1000;
        instance.rotationPartZylinderKipp.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartZylinderKipp#touchRotLimit"), 10));
    end;
	
	local rotationPartNodeGelenk1 = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.rotationPartGelenk1#index"));
    if rotationPartNodeGelenk1 ~= nil then
        instance.rotationPartGelenk1 = {};
        instance.rotationPartGelenk1.node = rotationPartNodeGelenk1;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartGelenk1#minRot"));
        instance.rotationPartGelenk1.minRot = {};
        instance.rotationPartGelenk1.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartGelenk1.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartGelenk1.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartGelenk1#maxRot"));
        instance.rotationPartGelenk1.maxRot = {};
        instance.rotationPartGelenk1.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartGelenk1.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartGelenk1.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.rotationPartGelenk1.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartGelenk1#rotTime"), 2)*1000;
        instance.rotationPartGelenk1.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartGelenk1#touchRotLimit"), 10));
    end;
	
	local rotationPartNodeGelenk2 = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.rotationPartGelenk2#index"));
    if rotationPartNodeGelenk2 ~= nil then
        instance.rotationPartGelenk2 = {};
        instance.rotationPartGelenk2.node = rotationPartNodeGelenk2;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartGelenk2#minRot"));
        instance.rotationPartGelenk2.minRot = {};
        instance.rotationPartGelenk2.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartGelenk2.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartGelenk2.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartGelenk2#maxRot"));
        instance.rotationPartGelenk2.maxRot = {};
        instance.rotationPartGelenk2.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartGelenk2.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartGelenk2.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.rotationPartGelenk2.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartGelenk2#rotTime"), 2)*1000;
        instance.rotationPartGelenk2.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartGelenk2#touchRotLimit"), 10));
    end;
	
	local translationPartNodeZylinderKipp2 = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.translationPartZylinderKipp2#index"));
    if translationPartNodeZylinderKipp2 ~= nil then
        instance.translationPartZylinderKipp2 = {};
        instance.translationPartZylinderKipp2.node = translationPartNodeZylinderKipp2;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.translationPartZylinderKipp2#minTrans"));
        instance.translationPartZylinderKipp2.minTrans = {};
        instance.translationPartZylinderKipp2.minTrans[1] = Utils.getNoNil(x, 0);
        instance.translationPartZylinderKipp2.minTrans[2] = Utils.getNoNil(y, 0);
        instance.translationPartZylinderKipp2.minTrans[3] = Utils.getNoNil(z, 0);

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.translationPartZylinderKipp2#maxTrans"));
        instance.translationPartZylinderKipp2.maxTrans = {};
        instance.translationPartZylinderKipp2.maxTrans[1] = Utils.getNoNil(x, 0);
        instance.translationPartZylinderKipp2.maxTrans[2] = Utils.getNoNil(y, 0);
        instance.translationPartZylinderKipp2.maxTrans[3] = Utils.getNoNil(z, 0);

        instance.translationPartZylinderKipp2.transTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.translationPartZylinderKipp2#transTime"), 2)*1000;
        instance.translationPartZylinderKipp2.touchTransLimit = Utils.getNoNil(getXMLString(xmlFile, "vehicle.translationPartZylinderKipp2#touchTransLimit"), 10);
    end;
	
	local translationPartNodeZylinderHub2 = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.translationPartZylinderHub2#index"));
    if translationPartNodeZylinderHub2 ~= nil then
        instance.translationPartZylinderHub2 = {};
        instance.translationPartZylinderHub2.node = translationPartNodeZylinderHub2;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.translationPartZylinderHub2#minTrans"));
        instance.translationPartZylinderHub2.minTrans = {};
        instance.translationPartZylinderHub2.minTrans[1] = Utils.getNoNil(x, 0);
        instance.translationPartZylinderHub2.minTrans[2] = Utils.getNoNil(y, 0);
        instance.translationPartZylinderHub2.minTrans[3] = Utils.getNoNil(z, 0);

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.translationPartZylinderHub2#maxTrans"));
        instance.translationPartZylinderHub2.maxTrans = {};
        instance.translationPartZylinderHub2.maxTrans[1] = Utils.getNoNil(x, 0);
        instance.translationPartZylinderHub2.maxTrans[2] = Utils.getNoNil(y, 0);
        instance.translationPartZylinderHub2.maxTrans[3] = Utils.getNoNil(z, 0);

        instance.translationPartZylinderHub2.transTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.translationPartZylinderHub2#transTime"), 2)*1000;
        instance.translationPartZylinderHub2.touchTransLimit = Utils.getNoNil(getXMLString(xmlFile, "vehicle.translationPartZylinderHub2#touchTransLimit"), 10);
    end;
	
	local rotationPartNodeJoystick1 = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.rotationPartJoystick1#index"));
    if rotationPartNodeJoystick1 ~= nil then
        instance.rotationPartJoystick1 = {};
        instance.rotationPartJoystick1.node = rotationPartNodeJoystick1;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartJoystick1#minRot"));
        instance.rotationPartJoystick1.minRot = {};
        instance.rotationPartJoystick1.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartJoystick1.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartJoystick1.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartJoystick1#maxRot"));
        instance.rotationPartJoystick1.maxRot = {};
        instance.rotationPartJoystick1.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartJoystick1.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartJoystick1.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.rotationPartJoystick1.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartJoystick1#rotTime"), 2)*1000;
        instance.rotationPartJoystick1.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartJoystick1#touchRotLimit"), 10));
    end;
	
	local rotationPartNodeJoystick2 = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.rotationPartJoystick2#index"));
    if rotationPartNodeJoystick2 ~= nil then
        instance.rotationPartJoystick2 = {};
        instance.rotationPartJoystick2.node = rotationPartNodeJoystick2;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartJoystick2#minRot"));
        instance.rotationPartJoystick2.minRot = {};
        instance.rotationPartJoystick2.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartJoystick2.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartJoystick2.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartJoystick2#maxRot"));
        instance.rotationPartJoystick2.maxRot = {};
        instance.rotationPartJoystick2.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartJoystick2.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartJoystick2.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.rotationPartJoystick2.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartJoystick2#rotTime"), 2)*1000;
        instance.rotationPartJoystick2.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartJoystick2#touchRotLimit"), 10));
    end;
	
	local rotationPartNodeJoystick3 = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.rotationPartJoystick3#index"));
    if rotationPartNodeJoystick3 ~= nil then
        instance.rotationPartJoystick3 = {};
        instance.rotationPartJoystick3.node = rotationPartNodeJoystick3;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartJoystick3#minRot"));
        instance.rotationPartJoystick3.minRot = {};
        instance.rotationPartJoystick3.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartJoystick3.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartJoystick3.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartJoystick3#maxRot"));
        instance.rotationPartJoystick3.maxRot = {};
        instance.rotationPartJoystick3.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartJoystick3.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartJoystick3.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.rotationPartJoystick3.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartJoystick3#rotTime"), 2)*1000;
        instance.rotationPartJoystick3.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartJoystick3#touchRotLimit"), 10));
    end;
	
	local rotationPartNodeJoystick4 = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.rotationPartJoystick4#index"));
    if rotationPartNodeJoystick4 ~= nil then
        instance.rotationPartJoystick4 = {};
        instance.rotationPartJoystick4.node = rotationPartNodeJoystick4;

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartJoystick4#minRot"));
        instance.rotationPartJoystick4.minRot = {};
        instance.rotationPartJoystick4.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartJoystick4.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartJoystick4.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotationPartJoystick4#maxRot"));
        instance.rotationPartJoystick4.maxRot = {};
        instance.rotationPartJoystick4.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        instance.rotationPartJoystick4.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        instance.rotationPartJoystick4.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        instance.rotationPartJoystick4.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartJoystick4#rotTime"), 2)*1000;
        instance.rotationPartJoystick4.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotationPartJoystick4#touchRotLimit"), 10));
    end;
	
--frontlader Ende

--AP start
    
    instance.autoPilotAreaLeft = {};
    instance.autoPilotAreaLeft.available = getXMLBool(xmlFile, "vehicle.autoPilotAreas#left");
    instance.autoPilotAreaLeft.startOutside = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.autoPilotAreas.autoPilotAreaLeft#startOutside"));
    instance.autoPilotAreaLeft.widthOutside = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.autoPilotAreas.autoPilotAreaLeft#widthOutside"));
    instance.autoPilotAreaLeft.heightOutside = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.autoPilotAreas.autoPilotAreaLeft#heightOutside"));
    instance.autoPilotAreaLeft.startInside = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.autoPilotAreas.autoPilotAreaLeft#startInside"));
    instance.autoPilotAreaLeft.widthInside = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.autoPilotAreas.autoPilotAreaLeft#widthInside"));
    instance.autoPilotAreaLeft.heightInside = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.autoPilotAreas.autoPilotAreaLeft#heightInside"));
    instance.autoPilotAreaLeft.active = false;

    instance.autoPilotAreaRight = {};
    instance.autoPilotAreaRight.available = getXMLBool(xmlFile, "vehicle.autoPilotAreas#right");
    instance.autoPilotAreaRight.startOutside = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.autoPilotAreas.autoPilotAreaRight#startOutside"));
    instance.autoPilotAreaRight.widthOutside = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.autoPilotAreas.autoPilotAreaRight#widthOutside"));
    instance.autoPilotAreaRight.heightOutside = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.autoPilotAreas.autoPilotAreaRight#heightOutside"));
    instance.autoPilotAreaRight.startInside = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.autoPilotAreas.autoPilotAreaRight#startInside"));
    instance.autoPilotAreaRight.widthInside = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.autoPilotAreas.autoPilotAreaRight#widthInside"));
    instance.autoPilotAreaRight.heightInside = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.autoPilotAreas.autoPilotAreaRight#heightInside"));
    instance.autoPilotAreaRight.active = false;

    if instance.autoPilotAreaLeft.available or instance.autoPilotAreaRight.available then
        instance.autoPilotPresent = true;
    else
        instance.autoPilotPresent = false;
    end;   

    instance.autoPilotEnabled = false;
    instance.autoPilotDelayLeft = 0;
    instance.autoPilotDelayRight = 0;

    instance.turnDirection = 0;
    instance.autoRotateBackSpeedBackup = instance.autoRotateBackSpeed;

    instance.stoppedap = false;
    instance.villogo = false;
    instance.dker = false;
    instance.off = false;   
 	
--AP stop
	
	instance.trailerAttacherJoint1 = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.attacherJoint#index"));
    instance.trailerAttacherJoint2 = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.saddleAttacherJoint#index"));
	
	
	instance.worklightsActive = false;
	instance.brakelightsActive = false;
	instance.fuelWarnlightsActive = false;
	instance.brakeWarnlightsActive = false;
	instance.flashlightsLeft2Active = false;
	instance.flashlightsRight2Active = false;
	instance.drucklightsActive = false;
	instance.druckWarnlightsActive = false;
	
	instance.compressorSoundRunning = false;
	instance.compressorStopSoundRunning = false;
	instance.brakeSoundRunning = false;
	instance.brakeReleaseSoundRunning = false;
	instance.fuelWarnSoundRunning = false;
	instance.handbrakeSoundRunning = false;
	instance.gang0 = true;
	
	instance.delayOnLeft = 0;
	instance.delayOffLeft = 0;
	instance.delayOnRight = 0;
	instance.delayOffRight = 0;
	instance.delayOn = 0;
	instance.delayOff = 0;
	instance.delayWischer1 = 0;
	instance.delayWischer2 = 0;
	instance.delayFuelOn = 0;
	instance.delayFuelOff = 0;
	instance.delayFuelSoundOn = 0;
	instance.delayFuelSoundOff = 0;
	instance.druckLevel = 0;
	instance.delayDruckOn = 0;
	instance.delayDruckOff = 0;
	

    delete(xmlFile);

    --instance.steeringAngle = 0;
    instance.rotatedTime = 0;
    instance.firstTimeRun = false;
    instance.camIndex = 1;
    instance.isEntered = false;

    instance.lightsActive = false;
	instance.farlightsActive = false;
	instance.frontlightsActive = false;
    instance.lastPosition = nil; -- = {0,0,0};
    instance.lastSpeed = 0;
    instance.lastMovedDistance = 0;
    instance.speedDisplayDt = 0;
    instance.speedDisplayScale = 1;
    instance.isBroken = false;

    instance.lastSoundSpeed = 0;

    instance.time = 0;

    instance.showWaterWarning = false;

    instance.hasRefuelStationInRange = false;
    instance.doRefuel = false;
    instance:setFuelFillLevel(instance.fuelCapacity);

    instance.waterSplashSample = nil;

    instance.hudBasePoxX = 0.8325;
    instance.hudBasePoxY = (1-0.99);
    instance.hudBaseWidth = 0.16;
    instance.hudBaseHeight = 0.1625;
    instance.hudBaseOverlay = Overlay:new("hudBaseOverlay", "data/missions/hud_vehicle_base.png", instance.hudBasePoxX, instance.hudBasePoxY, instance.hudBaseWidth, instance.hudBaseHeight);
	instance.hudGearPoxX = 0.815;
    instance.hudGearPoxY = 1-1.05;
    instance.hudGearWidth = 0.200;
    instance.hudGearHeight = 0.300;
    instance.hudGearOverlay = Overlay:new("hudGearDisplay", "data/vehicles/steerable/JD6820/JDHud.png", instance.hudGearPoxX, instance.hudGearPoxY, instance.hudGearWidth, instance.hudGearHeight);

    instance.refuelSampleRunning = false;
    instance.refuelSample = createSample("reverseDriveSound");
    loadSample(instance.refuelSample, "data/maps/sounds/refuel.wav", false);

    instance.twinWheelsActive= false;
    instance.plowWheelsActive= false;

    return instance;
end;

function Vehicle4:delete()
    for i=1, table.getn(self.cameras) do
        self.cameras[i]:delete();
    end;

    if self.attachSound ~= nil then
        delete(self.attachSound);
    end;
    if self.motorSound ~= nil then
        delete(self.motorSound);
    end;
    if self.motorStartSound ~= nil then
        delete(self.motorStartSound);
    end;
    if self.reverseDriveSound ~= nil then
        delete(self.reverseDriveSound);
    end;
    if self.waterSplashSample ~= nil then
        delete(self.waterSplashSample);
    end;
    if self.refuelSample ~= nil then
        delete(self.refuelSample);
    end;
    
    if self.hudBaseOverlay then
        self.hudBaseOverlay:delete();
    end;

    delete(self.rootNode);
end;

function Vehicle4:setWorldPosition(x,y,z, xRot,yRot,zRot)
    setTranslation(self.rootNode, x,y,z);
    setRotation(self.rootNode, xRot,yRot,zRot);
end;

function Vehicle4:mouseEvent(posX, posY, isDown, isUp, button)
    if self.isEntered then
        self.cameras[self.camIndex]:mouseEvent(posX, posY, isDown, isUp, button);
    end;
end;

function Vehicle4:keyEvent(unicode, sym, modifier, isDown)
    if self.isEntered then
        if self.selectedImplement ~= 0 then
            self.attachedImplements[self.selectedImplement].object:keyEvent(unicode, sym, modifier, isDown);
        end;
        if self.attachedTrailer ~= nil then
            self.attachedTrailer:keyEvent(unicode, sym, modifier, isDown);
        end;
    end;
	  if isDown and sym == Input.KEY_4 then
		self.worklightsActive = not self.worklightsActive;
	  end;
	
	if sym == Input.KEY_9 then
		self.translationMaxAttacher = isDown;
	end;
	if sym == Input.KEY_0 then
		self.translationMinAttacher = isDown;
	end;
	if isDown and sym == Input.KEY_6 then
		self.flashLeft = not self.flashLeft;
		self.delayOnLeft = 10;
		self.flashRight = false;
		self.flash = false;
		self.flashlightsRightActive = false;
		self.flashlightsWarnerActive = false;
		self.flashlightsRight2Active = false;
	end;
	if isDown and sym == Input.KEY_8 then
		self.flashRight = not self.flashRight;
		self.delayOnRight = 10;
		self.flashLeft = false;
		self.flash = false;
		self.flashlightsLeftActive = false;
		self.flashlightsWarnerActive = false;
		self.flashlightsLeft2Active = false;
	end;
	if isDown and sym == Input.KEY_7 then
		self.flash = not self.flash;
		self.flashRight = false;
		self.flashLeft = false;
		self.delayOn = 20;
		self.flashlightsRightActive = false;
		self.flashlightsLeftActive = false;
		self.flashlightsRight2Active = false;
		self.flashlightsLeft2Active = false;
	end;
	if isDown and sym == Input.KEY_backslash then
		self.helpPanel = not self.helpPanel;
	end;
	if isDown and sym == Input.KEY_space then
		self.handbrake = not self.handbrake;
	end;
	if self.isAutomatic then
		if isDown and sym == Input.KEY_pageup then
			if self.reverseGear then
				self.neutral = true;
				self.reverseGear = false;
			elseif self.neutral then
				self.forward = true;
				self.neutral = false;
				self.gang1 = true;
			end;
		end;
		if isDown and sym == Input.KEY_pagedown then
			if self.forward then
				self.neutral = true;
				self.forward = false;
			elseif self.neutral then
				self.reverseGear = true;
				self.neutral = false;
				self.gang01 = true;
			end;
		end;
	end;
	if not self.isAutomatic then
		if isDown and sym == Input.KEY_pageup then
			if self.gang02 == true then
				self.gang01 = true;
				self.gang02 = false;
			elseif self.gang01 == true then
				self.gang0 = true;
				self.gang01 = false;
			elseif self.gang0 == true then
				self.reverse = false;
				self.gang1 = true;
				self.gang0 = false;
			elseif self.gang1 == true then
				self.gang2 = true;
				self.gang1 = false;
			elseif self.gang2 == true then
				self.gang3 = true;
				self.gang2 = false;
			elseif self.gang3 == true then
				self.gang4 = true;
				self.gang3 = false;
			elseif self.gang4 == true then
				self.gang5 = true;
				self.gang4 = false;
			elseif self.gang5 == true then
				self.gang6 = true;
				self.gang5 = false;
			end;
		end;
	
		if isDown and sym == Input.KEY_pagedown then
			if self.gang6 == true then
				self.gang5 = true;
				self.gang6 = false;
			elseif self.gang5 == true then
				self.gang4 = true;
				self.gang5 = false;
			elseif self.gang4 == true then
				self.gang3 = true;
				self.gang4 = false;
			elseif self.gang3 == true then
				self.gang2 = true;
				self.gang3 = false;
			elseif self.gang2 == true then
				self.gang1 = true;
				self.gang2 = false;
			elseif self.gang1 == true then
				self.gang0 = true;
				self.gang1 = false;
			elseif self.gang0 == true then
				self.gang01 = true;
				self.reverse = true;
				self.gang0 = false;
			elseif self.gang01 == true then
				self.gang02 = true;
				self.reverse = true;
				self.gang01 = false;
			end;
		end;
	end;
	if isDown and sym == Input.KEY_return then
		self.isAutomatic = not self.isAutomatic;
		self.neutral = true;
		self.reverseGear = false;
		self.forward = false;
	end;
	
	if sym == Input.KEY_KP_4 then
		self.rotationMaxJoystick1 = isDown;
		self.rotationMinZylinderHub = isDown;
		self.translationMinZylinderHub2 = isDown;
	end;
	if sym == Input.KEY_KP_7 then
		self.rotationMaxJoystick2 = isDown;
		self.rotationMaxZylinderHub = isDown;
		self.translationMaxZylinderHub2 = isDown;
	end;
	if sym == Input.KEY_KP_8 then
		self.rotationMaxJoystick3 = isDown;
		self.rotationMaxGelenk1 = isDown;
		self.rotationMaxGelenk2 = isDown;
		self.rotationMaxZylinderKipp = isDown;
		self.translationMaxZylinderKipp2 = isDown;
	end;
	if sym == Input.KEY_KP_5 then
		self.rotationMaxJoystick4 = isDown;
		self.rotationMinGelenk1 = isDown;
		self.rotationMinGelenk2 = isDown;
		self.rotationMinZylinderKipp = isDown;
		self.translationMinZylinderKipp2 = isDown;
	end;
	
	if self.frontLoaderAttachedObject ~= nil then
        self.frontLoaderAttachedObject:keyEvent(unicode, sym, modifier, isDown)
    end;

    local jointDesc = self.frontLoaderJointDesc;
    if sym == Input.KEY_KP_8 or sym == Input.KEY_KP_5 then
        jointDesc.moveDirection2 = 0;
        if isDown then
        	if sym == Input.KEY_KP_5 then
                jointDesc.moveDirection2 = 1;
        	elseif sym == Input.KEY_KP_8 then
                jointDesc.moveDirection2 = -1;
        	end;
        end;
    end;
	
    if sym == Input.KEY_KP_4 or sym == Input.KEY_KP_7 then
        jointDesc.moveDirection = 0;
        if isDown then
        	if sym == Input.KEY_KP_4 then
                jointDesc.moveDirection = 1;
				--self.rotationMaxLeft = isDown;
        	elseif sym == Input.KEY_KP_7 then
                jointDesc.moveDirection = -1;
				--self.rotationMinLeft = isDown;
        	end;
        end;
    end;

    if isDown and sym == Input.KEY_KP_0 then
        if self.frontLoaderAttachedObject ~= nil then
            -- detach attachable
			self.frontLoaderAttachedObject:onDeselect();
            self.frontLoaderAttachedObject:onDetach();
			removeJoint(jointDesc.jointIndex);
            jointDesc.jointIndex = 0;
            self.frontLoaderAttachedObject = nil;
        else
            local attachableInRange = self:frontLoaderAttachableInRange();
            if attachableInRange ~= nil then
                -- attach
                local implement = {};
                self.frontLoaderAttachedObject = attachableInRange;
				self.frontLoaderAttachedObject:onAttach(self);
                self.frontLoaderAttachedObject:onSelect();


                local constr = JointConstructor:new();
                constr:setActors(self.rootNode, attachableInRange.rootNode);
                constr:setJointTransforms(jointDesc.jointTransform, attachableInRange.attacherJoint);

                for i=1, 3 do
                    constr:setRotationLimit(i-1, 0, 0);
                    constr:setTranslationLimit(i-1, true, 0, 0);
                end;
                jointDesc.jointIndex = constr:finalize();
                jointDesc.moveDirection = 0;
                jointDesc.moveDirection2 = 0;
            end;
        end;
    end;

   if isDown and sym == Input.KEY_t then 
		self.twinWheelsActive = not self.twinWheelsActive;
		self.plowWheelsActive = false;
   end;
   if isDown and sym == Input.KEY_y then 
		self.plowWheelsActive = not self.plowWheelsActive;
		self.twinWheelsActive = false;
   end;

   if self.isEntered then
      if self.rundumleuchtenKey ~= nil then
         if isDown and sym == Input[self.rundumleuchtenKey] then
           self.rundumleuchtenAn = not self.rundumleuchtenAn;
           for i=1, self.rundumleuchtenAnz do
             setVisibility(self.rundumleuchten[i].light, self.rundumleuchtenAn);
           end;
         end;
      end;
    end;

   if self.isEntered then
      if self.bremzleuchtenKey ~= nil then
         if isDown and sym == Input[self.bremzleuchtenKey] then
           for i=1, self.bremzleuchtenAnz do
             setVisibility(self.bremzleuchten[i].light, true);
           end;
	   else 
	     for i=1, self.bremzleuchtenAnz do
             setVisibility(self.bremzleuchten[i].light, false);
           end;
         end;
      end;
    end;

	
  if self.isEntered then
      if self.frontleuchtenKey ~= nil then
         if isDown and sym == Input[self.frontleuchtenKey] then
            if self.leuchtenAn == false and self.frontleuchtenAn == false and self.farleuchtenAn == false then
           		self.leuchtenAn = true;
			self.frontleuchtenAn = true;
		elseif self.frontleuchtenAn == true then
			self.frontleuchtenAn = false;
			self.farleuchtenAn = true;
		elseif self.farleuchtenAn == true then
			self.farleuchtenAn = false;
			self.leuchtenAn = false;
		end;
         end;
      end;
   end;

   if self.isEntered then
         for i=1, self.leuchtenAnz do
           setVisibility(self.leuchten[i].light, self.leuchtenAn);
         end;
    end;

   if self.isEntered then
         for i=1, self.frontleuchtenAnz do
           setVisibility(self.frontleuchten[i].light, self.frontleuchtenAn);
         end;
    end;

    if self.isEntered then
         for i=1, self.farleuchtenAnz do
           setVisibility(self.farleuchten[i].light, self.farleuchtenAn);
         end;
    end;

--AP start
	if sym == Input.KEY_x and isDown then
		self.villogo = not self.villogo;
                setVisibility(self.fL.node,self.villogo);			
    	end;

    	if sym == Input.KEY_z and isDown then
		self.dker = not self.dker;
		self.valt=true;
   	else 	             
        	self.valt=false;
    	end;
   

    
        if self.autoPilotPresent then
            if sym == Input.KEY_p and isDown then
                self.autoPilotEnabled = not self.autoPilotEnabled;

                if not self.autoPilotAreaLeft.active and not self.autoPilotAreaRight.active then
                    if not self.autoPilotAreaLeft.available then
                        self.autoPilotAreaLeft.active = true;
                    elseif self.autoPilotAreaRight.available then
                        self.autoPilotAreaRight.active = true;
                    end;
                end;

                if self.autoPilotEnabled then
                    self.autoRotateBackSpeed = 0;
                    playSample(self.warningSound, 3 , 10.6 , 0);
                else
                    self.autoRotateBackSpeed = self.autoRotateBackSpeedBackup;
                    playSample(self.warningSound, 1, 10.6, 0);
                end;
            end;

            if sym == Input.KEY_l and isDown then
                self.autoPilotAreaLeft.active = self.autoPilotAreaLeft.available;
                self.autoPilotAreaRight.active = false;

                self.autoPilotDelayLeft = 0;
                self.autoPilotDelayRight = 0;
            end;

            if sym == Input.KEY_r and isDown then
                self.autoPilotAreaRight.active = self.autoPilotAreaRight.available;
                self.autoPilotAreaLeft.active = false;

                self.autoPilotDelayRight = 0;
                self.autoPilotDelayLeft = 0;
            end;

        end;

--AP ende



end;

function Vehicle4:update(dt, isActive)

    self.time = self.time + dt;
	
	if self.isEntered and self.attachedTrailer then
		g_currentMission.isExtendedVehicle = true;
	else
		g_currentMission.isExtendedVehicle = false;
	end;
	
    if self.isEntered then

        local waterY = 79;
        xt,yt,zt = getTranslation(self.rootNode);
        local deltaWater = yt-waterY;
        if deltaWater < 0 then
            self.isBroken = true;
            g_currentMission:onSunkVehicle();
            g_currentMission:onLeaveVehicle();

            local volume = math.min(1, self.lastSpeed*3600/30);
            if self.waterSplashSample == nil then
                self.waterSplashSample = createSample("waterSplashSample");
                loadSample(self.waterSplashSample, "data/maps/sounds/waterSplash.wav", false);
            end;
            playSample(self.waterSplashSample, 1, volume, 0);

        end;
        self.showWaterWarning = deltaWater < 2;

        if self.playMotorSound and self.motorSound ~= nil and self.playMotorSoundTime <= self.time then
            playSample(self.motorSound, 0, 1, 0);
            self.playMotorSound = false;
        end;

        self.speedDisplayDt = self.speedDisplayDt + dt;
        if self.speedDisplayDt > 100 then
            local newX, newY, newZ = getWorldTranslation(self.rootNode);
            if self.lastPosition == nil then
                self.lastPosition = {newX, newY, newZ};
            end;
            self.lastMovedDistance = Utils.vector3Length(newX-self.lastPosition[1], newY-self.lastPosition[2], newZ-self.lastPosition[3]);
            self.lastSpeed = self.lastSpeed*0.3 + (self.lastMovedDistance/self.speedDisplayDt*self.speedDisplayScale)*0.7;
            self.lastPosition = {newX, newY, newZ};
            self.speedDisplayDt = 0;
        end;

        local fuelUsed = (self.motor.lastMotorRpm/1000)*self.fuelUsage;
        self:setFuelFillLevel(self.fuelFillLevel-fuelUsed);

        g_currentMission.missionStats.fuelUsageTotal = g_currentMission.missionStats.fuelUsageTotal + fuelUsed;
        g_currentMission.missionStats.fuelUsageSession = g_currentMission.missionStats.fuelUsageSession + fuelUsed;
        
        g_currentMission.missionStats.traveledDistanceTotal = g_currentMission.missionStats.traveledDistanceTotal + self.lastMovedDistance*0.001;
        g_currentMission.missionStats.traveledDistanceSession = g_currentMission.missionStats.traveledDistanceSession + self.lastMovedDistance*0.001;

        if InputBinding.hasEvent(InputBinding.SWITCH_IMPLEMENT) then
            local selected = self.selectedImplement;
            local numImplements = table.getn(self.attachedImplements);
            if selected ~= 0 and numImplements > 1 then
                selected = selected+1;
                if selected > numImplements then
                    selected = 1;
                end;
                self:setSelectedImplement(selected);
            end;
        end;

        if not g_currentMission.fixedCamera then
            if InputBinding.hasEvent(InputBinding.CAMERA_SWITCH) then
                self.cameras[self.camIndex]:onDeactivate();
                self.camIndex = self.camIndex + 1;
                if self.camIndex > self.numCameras then
                    self.camIndex = 1;
                end;
                self.cameras[self.camIndex]:onActivate();
            end;
        end;
        if InputBinding.hasEvent(InputBinding.TOGGLE_LIGHTS) then
            if self.lightsActive == false and self.farlightsActive == false and self.frontlightsActive == false then
				self.lightsActive = true;
				self.frontlightsActive = true;
			elseif self.frontlightsActive == true then
				self.farlightsActive = true;
				self.frontlightsActive = false;
			elseif self.farlightsActive == true then
				self.farlightsActive = false;
				self.lightsActive = false;
			end;
        end;
		if self.lightsActive then
			for i=1, self.numLights do
				local light = self.lights[i];
				setVisibility(light, self.lightsActive);
			end;
		else
			for i=1, self.numLights	do
				local light = self.lights[i];
				setVisibility(light, self.lightsActive, false);
			end;
		end;
		
		if self.frontlightsActive then
			for i=1, self.numFrontlights do
				local frontlight = self.frontlights[i];
				setVisibility(frontlight, self.frontlightsActive);
			end;
			setVisibility(self.lowLightDisplay, self.frontlightsActive);
		else
			for i=1, self.numFrontlights	do
				local frontlight = self.frontlights[i];
				setVisibility(frontlight, self.frontlightsActive, false);
			end;
			setVisibility(self.lowLightDisplay, false);
		end;
		
		if self.farlightsActive then
			for i=1, self.numFarlights do
				local farlight = self.farlights[i];
				setVisibility(farlight, self.farlightsActive);
			end;
			setVisibility(self.farLightDisplay, self.farlightsActive);
		else
			for i=1, self.numFarlights	do
				local farlight = self.farlights[i];
				setVisibility(farlight, self.farlightsActive, false);
			end;
			setVisibility(self.farLightDisplay, false);
		end;

		if self.isEntered then
		  if self.rundumleuchtenAn == true then
			setVisibility(self.roundLightDisplay, true);
		  else
			setVisibility(self.roundLightDisplay, false);
	    	  end;
    		end;
		if self.isEntered then
		  if self.frontleuchtenAn == true then
			setVisibility(self.lowLightDisplay, true);
		  else
			setVisibility(self.lowLightDisplay, false);
	    	  end;
    		end;
		if self.isEntered then
		  if self.farleuchtenAn == true then
			setVisibility(self.farLightDisplay, true);
		  else
			setVisibility(self.farLightDisplay, false);
	    	  end;
    		end;
		if self.isEntered then
		  if self.leuchtenAn == true then
			setVisibility(self.lightDisplay, true);
		  else
			setVisibility(self.lightDisplay, false);
	    	  end;
    		end;

		

        if self.doRefuel and InputBinding.hasEvent(InputBinding.REFUEL) then
            self.doRefuel = false;
        else
            if self.hasRefuelStationInRange and InputBinding.hasEvent(InputBinding.REFUEL) then
                self.doRefuel = true;
            end;
        end;

        if InputBinding.hasEvent(InputBinding.SPEED_LEVEL1) then
            self.motor:setSpeedLevel(1, false);
        elseif InputBinding.hasEvent(InputBinding.SPEED_LEVEL2) then
            self.motor:setSpeedLevel(2, false);
        elseif InputBinding.hasEvent(InputBinding.SPEED_LEVEL3) then
            self.motor:setSpeedLevel(3, false);
        end;

        if InputBinding.hasEvent(InputBinding.ATTACH) then
            self:handleAttachEvent();
        end;

        if InputBinding.hasEvent(InputBinding.LOWER_IMPLEMENT) then
            self:handleLowerImplementEvent();
        end;

        --if self.attachedAttachable ~= nil then
        for i=1, table.getn(self.attachedImplements) do
            self.attachedImplements[i].object:update(dt);
        end;
    end;

    if self.isEntered then

        local acceleration = 0;
        if g_currentMission.allowSteerableMoving and not self.playMotorSound then
           
			if self.reverse then
				if self.gang01 then
					if getInputAxis(Input.AXIS_Y) == -1 or self.motor.speedLevel ~= 0 then
						acceleration = -0.06;
					end;
				elseif self.gang02 then
					if getInputAxis(Input.AXIS_Y) == -1 or self.motor.speedLevel ~= 0 then
						acceleration = -0.2;
					end;
				end;
			else
				if self.gang0 then
					if getInputAxis(Input.AXIS_Y) == -1 or self.motor.speedLevel ~= 0 then
						acceleration = 0;
					end;
				elseif self.gang1 then
					if getInputAxis(Input.AXIS_Y) == -1 or self.motor.speedLevel ~= 0 then
						acceleration = 0.05;
					end;
				elseif self.gang2 then
					if getInputAxis(Input.AXIS_Y) == -1 or self.motor.speedLevel ~= 0 then
						acceleration = 0.1;
					end;
				elseif self.gang3 then
					if getInputAxis(Input.AXIS_Y) == -1 or self.motor.speedLevel ~= 0 then
						acceleration = 0.2;
					end;
				elseif self.gang4 then
					if getInputAxis(Input.AXIS_Y) == -1 or self.motor.speedLevel ~= 0 then
						acceleration = 0.3;
					end;
				elseif self.gang5 then
					if getInputAxis(Input.AXIS_Y) == -1 or self.motor.speedLevel ~= 0 then
						if self.motor.lastMotorRpm < 1100 then
							acceleration = 0.35;
						elseif self.motor.lastMotorRpm > 1100 then
							acceleration = 0.55;
						end;
					end;
				elseif self.gang6 then
					if getInputAxis(Input.AXIS_Y) == -1 or self.motor.speedLevel ~= 0 then
						if self.motor.lastMotorRpm < 1100 then
							acceleration = 0.68;
						elseif self.motor.lastMotorRpm > 1100 then
							acceleration = 1.0;
						end;
					end;
				end;
			end;
			
			if getInputAxis(Input.AXIS_Y) == 1 then
				self.motor:setSpeedLevel(0, true)
			end;
			if getInputAxis(Input.AXIS_Y) == -1 then
				self.motor:setSpeedLevel(0,true)
			end;
        end;
        if self.fuelFillLevel == 0 then
            acceleration = 0;
        end;

        local worldX,worldY,worldZ = localDirectionToWorld(self.rootNode, 0, -self.downForce*dt/1000, 0);
        addForce(self.rootNode, worldX, worldY, worldZ, 0, 0, 0, true);


        --if Input.isKeyPressed(Input.KEY_a) then
        local rotScale = math.min(1.0/(self.lastSpeed*50+1), 1);
        local inputAxisX = getInputAxis(0);
        if inputAxisX < 0 then
            self.rotatedTime = math.min(self.rotatedTime - dt/1000*inputAxisX*rotScale, self.maxRotTime);
            --if self.rotatedTime > self.maxRotTime then
            --    self.rotatedTime = self.maxRotTime;
            --end;
        --elseif Input.isKeyPressed(Input.KEY_d) then
        elseif inputAxisX > 0 then
            self.rotatedTime = math.max(self.rotatedTime - dt/1000*inputAxisX*rotScale, self.minRotTime);
            --if self.rotatedTime < self.minRotTime then
            --    self.rotatedTime = self.minRotTime;
            --end;
        else
            if self.autoRotateBackSpeed ~= 0 then
                if self.rotatedTime > 0 then
                    self.rotatedTime = math.max(self.rotatedTime - dt/1000*self.autoRotateBackSpeed*rotScale, 0);
                else
                    self.rotatedTime = math.min(self.rotatedTime + dt/1000*self.autoRotateBackSpeed*rotScale, 0);
                end;
            end;
        end;

        if self.firstTimeRun then
            WheelsUtil.updateWheels(self, dt, self.lastSpeed, acceleration, false, 0)
        end;

        if self.numWheels > 0 then
            if self.motorSound ~= nil then
                --setSamplePitch(self.motorSound, math.min(self.motorSoundPitchOffset + self.motorSoundPitchScale*math.abs(self.lastSoundSpeed), self.motorSoundPitchMax));
                local roundPerSecond = (self.motor.lastMotorRpm-self.motor.minRpm) / 60;
                setSamplePitch(self.motorSound, math.min(self.motorSoundPitchOffset + self.motorSoundPitchScale*math.abs(roundPerSecond), self.motorSoundPitchMax));
                --print(math.min(self.motorSoundPitchOffset + self.motorSoundPitchScale*math.abs(roundPerSecond), self.motorSoundPitchMax));
            end;
        end;
        
        if self.steering ~= nil then
            setRotation(self.steering, 0, self.rotatedTime*self.steeringSpeed, 0);
        end;

		local rpm = math.max(self.motor.lastMotorRpm/50,0);
		local rpmDeltaMotion = math.max(self.motor.lastMotorRpm/300,0);
             
	      if self.incrementRotation == true then 
        		self.currentRotation = self.currentRotation + self.exhaustRotationSpeed;
	 	else
			self.currentRotation = self.currentRotation - self.exhaustRotationSpeed;
	 	end;

	  	if self.currentRotation >= (self.exhaustRotationMax - rpmDeltaMotion)then
			self.incrementRotation = false;
	 	elseif self.currentRotation <= (self.exhaustRotationMin + rpmDeltaMotion) then
  			self.incrementRotation = true;
	  	end;
	      setRotation(self.exhaustCover,Utils.degToRad(self.currentRotation+rpm), Utils.degToRad(180), 0);

		self.fanRotation = self.fanRotation + 15 + math.max(self.motor.lastMotorRpm/50,0);
		setRotation(self.fan, 0, 0, Utils.degToRad(self.fanRotation));

        self.doRefuel = self.doRefuel and self.hasRefuelStationInRange;
        if self.doRefuel then
        
            if not self.refuelSampleRunning then
                --print("start");
                playSample(self.refuelSample, 0, 1, 0);
                self.refuelSampleRunning = true;
            end;
        
            local refuelSpeed = 0.01;
            local currentFillLevel = self.fuelFillLevel;
            self:setFuelFillLevel(self.fuelFillLevel+refuelSpeed*dt);
            local delta = (self.fuelFillLevel-currentFillLevel)*g_fuelPricePerLiter;
            
            --print("delta ", delta);
            
            if delta <= 0 then
                self.doRefuel = false;
            end;
            
            g_currentMission.missionStats.expensesTotal = g_currentMission.missionStats.expensesTotal + delta;
            g_currentMission.missionStats.expensesSession = g_currentMission.missionStats.expensesSession + delta;
            g_currentMission.missionStats.money = g_currentMission.missionStats.money - delta;

        else
            if self.refuelSampleRunning then
                --print("stop");
                stopSample(self.refuelSample);
                self.refuelSampleRunning = false;
            end;
        end;
		
		if self.isAutomatic then
			if self.forward then
				self.reverse = false;
				if self.gang6 then
					if self.motor.lastMotorRpm < 1000 then
						self.gang5 = true;
						self.gang6 = false;
					end;
				elseif self.gang5 then
					if self.motor.lastMotorRpm < 950 then
						self.gang4 = true;
						self.gang5 = false;
					elseif self.motor.lastMotorRpm > 1785 then
						self.gang6 = true;
						self.gang5 = false;
					end;
				elseif self.gang4 then
					if self.motor.lastMotorRpm < 950 then
						self.gang3 = true;
						self.gang4 = false;
					elseif self.motor.lastMotorRpm > 1750 then
						self.gang5 = true;
						self.gang4 = false;
					end;
				elseif self.gang3 then
					if self.motor.lastMotorRpm < 950 then
						self.gang2 = true;
						self.gang3 = false;
					elseif self.motor.lastMotorRpm > 1700 then
						self.gang4 = true;
						self.gang3 = false;
					end;
				elseif self.gang2 then
					if self.motor.lastMotorRpm < 950 then
						self.gang1 = true;
						self.gang2 = false;
					elseif self.motor.lastMotorRpm > 1700 then
						self.gang3 = true;
						self.gang2 = false;
					end;
				elseif self.gang1 then
					if self.motor.lastMotorRpm > 1700 then
						self.gang2 = true;
						self.gang1 = false;
					end;
				end;
				self.gang0 = false;			
			end;
			if self.neutral then
				self.gang0 = true;
				self.gang01 = false;
				self.gang02 = false;
				self.gang1 = false;
				self.gang2 = false;
				self.gang3 = false;
				self.gang4 = false;
				self.gang5 = false;
				self.gang6 = false;
				self.reverse = false;
			end;
			if self.reverseGear then
				self.reverse = true;
				if self.gang02 then
					if self.motor.lastMotorRpm < 600 then
						self.gang01 = true;
						self.gang02 = false;
					end;
				elseif self.gang01 then
					if self.motor.lastMotorRpm > 1750 then
						self.gang02 = true;
						self.gang01 = false;
					end;
				end;
				self.gang0 = false;
			end;
		end;
	end;

    _=[[local axleSpeedSum = 0;
    self.lastWheelRpm = 0;
    for i=1, self.numWheels do
        local steeringAngle = 0;
        if self.wheels[i].rotSpeed ~= 0 then
            steeringAngle = self.rotatedTime * self.wheels[i].rotSpeed;
            if steeringAngle > self.wheels[i].rotMax then
                steeringAngle = self.wheels[i].rotMax;
            elseif steeringAngle < self.wheels[i].rotMin then
                steeringAngle = self.wheels[i].rotMin;
            end;
        end;
        local actMotorTorque = motorTorque;
        if self.wheels[i].driveMode == 0 then
            actMotorTorque = 0;
        end;
        setWheelShapeProps(self.wheels[i].node, self.wheels[i].wheelShape, actMotorTorque, brakeForce, steeringAngle);

        local x,y,z = getRotation(self.wheels[i].repr);
        local xDrive,yDrive,zDrive;
        if self.wheels[i].repr == self.wheels[i].driveNode then
            xDrive,yDrive,zDrive = x,y,z;
        else
            xDrive,yDrive,zDrive = getRotation(self.wheels[i].driveNode);
        end;
        if self.firstTimeRun then
            local newX, newY, newZ = getWheelShapePosition(self.wheels[i].node, self.wheels[i].wheelShape);
            setTranslation(self.wheels[i].repr, newX, newY, newZ);
            local axleSpeedDeg = getWheelShapeAxleSpeed(self.wheels[i].node, self.wheels[i].wheelShape);
            local axleSpeed = axleSpeedDeg*3.14159/180;
            axleSpeedSum = axleSpeedSum + axleSpeed;
            self.lastWheelRpm = self.lastWheelRpm + math.abs(axleSpeed)/(math.pi*2) * 60; --math.abs(axleSpeedDeg)/360 * 60;
            --print("axleSpeed", i, ": ", axleSpeed);
            --rotate(self.wheels[i].repr, axleSpeed*dt/1000.0, 0, 0);
            xDrive = xDrive+axleSpeed*dt/1000.0;

            --2*pi*hz = w   rad/s
            --hz = w/(2pi)
        end;
        if self.wheels[i].repr == self.wheels[i].driveNode then
            setRotation(self.wheels[i].repr, xDrive, steeringAngle, z);
        else
            setRotation(self.wheels[i].repr, x, steeringAngle, z);
            setRotation(self.wheels[i].driveNode, xDrive, yDrive, zDrive);
        end;
    end;
    self.lastWheelRpm = self.lastWheelRpm / self.numWheels;
    if axleSpeedSum > 0.01 then
        self.movingDirection = 1;
    elseif axleSpeedSum < -0.01 then
        self.movingDirection = -1;
    else
        self.movingDirection = 0;
    end;

    self.lastSoundSpeed = math.abs(axleSpeedSum/self.numWheels);]]

    --if self.attachedAttachable ~= nil then
    for i=1, table.getn(self.attachedImplements) do
        local implement = self.attachedImplements[i];
        local jointDesc = self.attacherJoints[implement.jointDescIndex];
        --local jointDesc = self.attacherJoints[self.attachedAttachableIndex];
        if jointDesc.topArm ~= nil and implement.object.topReferenceNode ~= nil then
            local ax, ay, az = getWorldTranslation(jointDesc.topArm.rotationNode);
            local bx, by, bz = getWorldTranslation(implement.object.topReferenceNode);

            local x, y, z = worldDirectionToLocal(getParent(jointDesc.topArm.rotationNode), bx-ax, by-ay, bz-az);
            setDirection(jointDesc.topArm.rotationNode, x*jointDesc.topArm.zScale, y*jointDesc.topArm.zScale, z*jointDesc.topArm.zScale, 0, 1, 0);
            if jointDesc.topArm.translationNode ~= nil then
                local distance = Utils.vector3Length(ax-bx, ay-by, az-bz);
                setTranslation(jointDesc.topArm.translationNode, 0, 0, (distance-jointDesc.topArm.referenceDistance)*jointDesc.topArm.zScale);
            end;
        end;
        if jointDesc.bottomArm ~= nil and implement.object.topReferenceNode ~= nil then
            local ax, ay, az = getWorldTranslation(jointDesc.bottomArm.rotationNode);
            local bx, by, bz = getWorldTranslation(implement.object.attacherJoint);

            local x, y, z = worldDirectionToLocal(getParent(jointDesc.bottomArm.rotationNode), bx-ax, by-ay, bz-az);
            setDirection(jointDesc.bottomArm.rotationNode, x*jointDesc.bottomArm.zScale, y*jointDesc.bottomArm.zScale, z*jointDesc.bottomArm.zScale, 0, 1, 0);
            if jointDesc.bottomArm.translationNode ~= nil then
                local distance = Utils.vector3Length(ax-bx, ay-by, az-bz);
                setTranslation(jointDesc.bottomArm.translationNode, 0, 0, (distance-jointDesc.bottomArm.referenceDistance)*jointDesc.bottomArm.zScale);
            end;
        end;
    end;

    if self.isEntered then
        if not g_currentMission.fixedCamera then
            setCamera(self.cameras[self.camIndex].cameraNode);
            self.cameras[self.camIndex]:update(dt);
        else
            -- set fixed tip camera
            if self.tipCamera ~= nil then
                setCamera(self.tipCamera);
            else
                self.cameras[self.camIndex]:resetCamera();
            end;
        end;
        --setCamera(getChildAt(self.rootNode, self.cameras[self.camIndex]));

        --if self.attachedAttachable ~= nil then
        for i=1, table.getn(self.attachedImplements) do
            local implement = self.attachedImplements[i];
            local jointDesc = self.attacherJoints[implement.jointDescIndex];
            --local jointDesc = self.attacherJoints[self.attachedAttachableIndex];

            local jointFrameInvalid = false;
            if jointDesc.rotationNode ~= nil then
                local x, y, z = getRotation(jointDesc.rotationNode);
                local rot = {x,y,z};
                local newRot = Utils.getMovedLimitedValues(rot, jointDesc.maxRot, jointDesc.minRot, 3, jointDesc.moveTime, dt, not jointDesc.moveDown);
                setRotation(jointDesc.rotationNode, unpack(newRot));
                for i=1, 3 do
                    if math.abs(newRot[i] - rot[i]) > 0.001 then
                        jointFrameInvalid = true;
                    end;
                end;
            end;
            if jointDesc.rotationNode2 ~= nil then
                local x, y, z = getRotation(jointDesc.rotationNode2);
                local rot = {x,y,z};
                local newRot = Utils.getMovedLimitedValues(rot, jointDesc.maxRot2, jointDesc.minRot2, 3, jointDesc.moveTime, dt, not jointDesc.moveDown);
                setRotation(jointDesc.rotationNode2, unpack(newRot));
                for i=1, 3 do
                    if math.abs(newRot[i] - rot[i]) > 0.001 then
                        jointFrameInvalid = true;
                    end;
                end;
            end;
            if jointFrameInvalid then
                setJointFrame(jointDesc.jointIndex, 0, jointDesc.jointTransform);
            end;

            local newRotLimit = {0,0,0};
            if not implement.object.fixedAttachRotation then
                newRotLimit = Utils.getMovedLimitedValues(implement.jointRotLimit, jointDesc.maxRotLimit, {0,0,0}, 3, jointDesc.moveTime, dt, not jointDesc.moveDown);
            end;
            for i=1, 3 do
                if math.abs(newRotLimit[i] - implement.jointRotLimit[i]) > 0.001 then
                    setJointRotationLimit(jointDesc.jointIndex, i-1, true, -newRotLimit[i], newRotLimit[i]);
                end;
            end;
            implement.jointRotLimit = newRotLimit;

            local newTransLimit = Utils.getMovedLimitedValues(implement.jointTransLimit, jointDesc.maxTransLimit, {0,0,0}, 3, jointDesc.moveTime, dt, not jointDesc.moveDown);
            for i=1, 3 do
                if math.abs(newTransLimit[i] - implement.jointTransLimit[i]) > 0.001 then
                    setJointTranslationLimit(jointDesc.jointIndex, i-1, true, -newTransLimit[i], newTransLimit[i]);
                end;
            end;
            implement.jointTransLimit = newTransLimit;
        end;

        if self.reverseDriveSound ~= nil and self.movingDirection == -1 then
            if not self.reverseDriveSoundEnabled then
                playSample(self.reverseDriveSound, 0, 0.7, 0);
                self.reverseDriveSoundEnabled = true;
            end;
        else
            if self.reverseDriveSoundEnabled then
                stopSample(self.reverseDriveSound);
                self.reverseDriveSoundEnabled = false;
            end;
        end;

        -- update crops
        _=[[for i=1, table.getn(self.cuttingAreas) do
            local worldX, worldY, worldZ = localToWorld(self.rootNode, self.cuttingAreas[i].startX, 0, self.cuttingAreas[i].startZ);
            local worldX2, worldY2, worldZ2 = localToWorld(self.rootNode, self.cuttingAreas[i].startX+self.cuttingAreas[i].width, 0, self.cuttingAreas[i].startZ);
            local worldX3, worldY3, worldZ3 = localToWorld(self.rootNode, self.cuttingAreas[i].startX, 0, self.cuttingAreas[i].startZ+self.cuttingAreas[i].height);
            Utils.updateWheatArea(worldX, worldZ, worldX2, worldZ2, worldX3, worldZ3, 0.0);
        end;]]
    end;
	
	
			
--John Deere Auspufqualm und Compressor Sound Beginn
	if self.isEntered then
		if getInputAxis(Input.AXIS_Y) == -1 then
			if self.motor.lastMotorRpm > 320 and self.motor.lastMotorRpm < 1000 then
				if self.attachedTrailer then
					Utils.setEmittingState(self.exhaustParticleSystems2, true);
				else
					Utils.setEmittingState(self.exhaustParticleSystems2, false);
				end;
				Utils.setEmittingState(self.exhaustParticleSystems4, true);
			else
				Utils.setEmittingState(self.exhaustParticleSystems4, false);
				Utils.setEmittingState(self.exhaustParticleSystems2, false);
			end;
		else
			Utils.setEmittingState(self.exhaustParticleSystems4, false);
			Utils.setEmittingState(self.exhaustParticleSystems2, false);
		end;
		if getInputAxis(Input.AXIS_Y) == -1 then
			if self.motor.lastMotorRpm > 1000 then
				if self.attachedTrailer then
					Utils.setEmittingState(self.exhaustParticleSystems3, true);
				else
					Utils.setEmittingState(self.exhaustParticleSystems3, false);
				end;
				Utils.setEmittingState(self.exhaustParticleSystems5, true);
			else
				Utils.setEmittingState(self.exhaustParticleSystems5, false);
				Utils.setEmittingState(self.exhaustParticleSystems3, false);
			end;
		else
			Utils.setEmittingState(self.exhaustParticleSystems5, false);
			Utils.setEmittingState(self.exhaustParticleSystems3, false);
		end;
	end;
--John Deere Auspuffqualm und Compressor Sound Ende
--Helppanel Beginn
	if self.isEntered then
		if not self.helpPanel then
			renderText(0.02, 0.2, 0.021, "Key #: Show Info");
		else
			renderText(0.02, 0.2, 0.021, "Key #: Dont show info");
		end;
		if self.helpPanel then
			renderText(0.02, 0.44, 0.023, "- John Deere 8620 Info -");
			if self.twinWheelsActive then
				renderText(0.02, 0.42, 0.021, "Hit T: Dismount twin-wheels");
			else
				renderText(0.02, 0.42, 0.021, "Hit T: Mount twin-wheels");
			end;
			if self.plowWheelsActive then
				renderText(0.02, 0.40, 0.021, "Hit Y: Dismount plow wheels");
			else
				renderText(0.02, 0.40, 0.021, "Hit Y: Mount plow wheels");
			end;
			if self.worklightsActive then
				renderText(0.02, 0.38, 0.021, "Hit 5: Turn off round light");
			else
				renderText(0.02, 0.38, 0.021, "Hit 5: Turn on round light");
			end;
			--if self.flashLeft then
				--renderText(0.02, 0.38, 0.021, "Hit 6: Blinker Links Abschalten");
			--else
				--renderText(0.02, 0.38, 0.021, "Hit 6: Blinker Links Einschalten");
			--end;
			--if self.flashRight then
				--renderText(0.02, 0.34, 0.021, "Hit 8: Blinker Rechts Abschalten");
			--else
				--renderText(0.02, 0.34, 0.021, "Hit 8: Blinker Rechts Einschalten");
			--end;
			--if self.flash then
				--renderText(0.02, 0.36, 0.021, "Hit 7: Warnblinker Abschalten");
			--else
				--renderText(0.02, 0.36, 0.021, "Hit 7: Warnblinker Einschalten");
			--end;
			if self.worklightsActive then
				renderText(0.02, 0.36, 0.021, "Hit 4: Turn off worklights");
			else
				renderText(0.02, 0.36, 0.021, "Hit 4: Turn on worklights");
			end;
			--renderText(0.02, 0.32, 0.021, "Hit 9/0: Zugmaul Hoch/Runter");
			renderText(0.02, 0.30, 0.021, "Hit space: Handbrake");
			renderText(0.02, 0.28, 0.021, "Hit PageUp/PageDown: Gear Up/Down");
			renderText(0.02, 0.26, 0.021, "Hit Enter: Change between Automatik/Manual gearbox");
			--renderText(0.02, 0.24, 0.021, "Hit NUM 8/5: Frontlader Kippen");
			--renderText(0.02, 0.22, 0.021, "Hit NUM 7/4: Frontlader Hoch/Runter");
			
		end;
	end;
--Helppanel Ende
--Gnge Beginn
	local ratio = 0;
	local rpmRange = 0;
	if self.gang02 then
		ratio = 6.5;
		rpmRange = 150;
		self.motor.backwardGearRatio = ratio;
		self.motor.forwardGearRatio = ratio;
		self.motor.rpmFadeOutRange = rpmRange;
		setRotation(self.gearShifter, Utils.degToRad(-8.673), Utils.degToRad(4.336), Utils.degToRad(4.805));
            setRotation(self.gearIndicator, Utils.degToRad(-36.173), Utils.degToRad(0), Utils.degToRad(0));
	elseif self.gang01 then
		ratio = 14;
		rpmRange = 300;
		self.motor.backwardGearRatio = ratio;
		self.motor.forwardGearRatio = ratio;
		self.motor.rpmFadeOutRange = rpmRange;
		setRotation(self.gearShifter, Utils.degToRad(-15.103), Utils.degToRad(4.336), Utils.degToRad(4.805));
		setRotation(self.gearIndicator, Utils.degToRad(-40.871), Utils.degToRad(0), Utils.degToRad(0));
	elseif self.gang0 then
		ratio = 0;
		self.motor.backwardGearRatio = ratio;
		self.motor.forwardGearRatio = ratio;
		setRotation(self.gearShifter, Utils.degToRad(-23.328), Utils.degToRad(5.284), Utils.degToRad(12.055));
		setRotation(self.gearIndicator, Utils.degToRad(-46.951), Utils.degToRad(0), Utils.degToRad(0));
	elseif self.gang1 then
		ratio = 15;
		rpmRange = 250;
		self.motor.forwardGearRatio = ratio;
		self.motor.rpmFadeOutRange = rpmRange;
		setRotation(self.gearShifter, Utils.degToRad(-22.025), Utils.degToRad(7.809), Utils.degToRad(13.667));
		setRotation(self.gearIndicator, Utils.degToRad(-31.471), Utils.degToRad(0), Utils.degToRad(0));
	elseif self.gang2 then
		ratio = 10;
		rpmRange = 200;
		self.motor.forwardGearRatio = ratio;
		self.motor.rpmFadeOutRange = rpmRange;
		setRotation(self.gearShifter, Utils.degToRad(-16.899), Utils.degToRad(7.809), Utils.degToRad(13.8));
		setRotation(self.gearIndicator, Utils.degToRad(-12.575), Utils.degToRad(0), Utils.degToRad(0));
	elseif self.gang3 then
		ratio = 7.5;
		rpmRange = 200;
		self.motor.forwardGearRatio = ratio;
		self.motor.rpmFadeOutRange = rpmRange;
		setRotation(self.gearShifter, Utils.degToRad(-11.528), Utils.degToRad(7.809), Utils.degToRad(14));
		setRotation(self.gearIndicator, Utils.degToRad(-3.313), Utils.degToRad(0), Utils.degToRad(0));
	elseif self.gang4 then
		ratio = 5.5;
		rpmRange = 200;
		self.motor.forwardGearRatio = ratio;
		self.motor.rpmFadeOutRange = rpmRange;
		setRotation(self.gearShifter, Utils.degToRad(-4), Utils.degToRad(7.809), Utils.degToRad(14.3));
		setRotation(self.gearIndicator, Utils.degToRad(5.885), Utils.degToRad(0), Utils.degToRad(0));
	elseif self.gang5 then
		ratio = 4.0;
		rpmRange = 150;
		self.motor.forwardGearRatio = ratio;
		self.motor.rpmFadeOutRange = rpmRange;
		setRotation(self.gearShifter, Utils.degToRad(1.495), Utils.degToRad(7.785), Utils.degToRad(14.608));
		setRotation(self.gearIndicator, Utils.degToRad(20.224), Utils.degToRad(0), Utils.degToRad(0));
	elseif self.gang6 then
		ratio = 2.7;
		rpmRange = 1;
		self.motor.forwardGearRatio = ratio;
		self.motor.rpmFadeOutRange = rpmRange;
		setRotation(self.gearShifter, Utils.degToRad(7.885), Utils.degToRad(7.596), Utils.degToRad(16.004));
		setRotation(self.gearIndicator, Utils.degToRad(32.669), Utils.degToRad(0), Utils.degToRad(0));
	end;
--Gnge Ende
--Blinkleuchten etc. Beginn
	if self.isEntered then
		if self.druckLevel >= -1 and self.druckLevel <= 6100 then
			self.compressorStart = true;
		end;
		if self.druckLevel >= 9999 then
			self.compressorStart = false;
		end;
		if self.compressorStart == true then
			if not self.compressorSoundRunning then
				playSample(self.compressorSound, 0, 1, 0);
				self.compressorSoundRunning = true;
			end;
			self.compressorStopSoundRunning = false;
			if self.druckLevel <= 10000 then
				self.druckLevel = self.druckLevel +(dt/3);
			end;
		end;
		if self.compressorStart == false then
			stopSample(self.compressorSound, 0, 1, 0);
			self.compressorSoundRunning = false;
			if not self.compressorStopSoundRunning then
				playSample(self.compressorStopSound, 1, 1, 0);
				self.compressorStopSoundRunning = true;
			end;
			if self.druckLevel >= 6099 then
				self.druckLevel = self.druckLevel -(dt/40);
			end;
		end;
		if self.druckLevel <= 6000 then
			self.druckWarning = true;
		else
			self.druckWarning = false;
			self.druckWarnlightsActive = false;
		end;
		if self.druckLevel >= 6000 then
			self.drucklightsActive = true;
		else
			self.drucklightsActive = false;
		end;
	end;
	
	if self.druckWarning then
		if self.delayDruckOn >= 0 then
			self.delayDruckOn = self.delayDruckOn -dt;
		end;
		if not self.druckWarnlightsActive then
			if self.delayDruckOn < 0 then
				self.delayDruckOff = 500;
				self.druckWarnlightsActive = true;
			end;
		end;
		if self.delayDruckOff >= 0 then
			self.delayDruckOff = self.delayDruckOff -dt;
		end;
		if self.druckWarnlightsActive then
			if self.delayDruckOff < 0 then
				self.delayDruckOn = 500;
				self.druckWarnlightsActive = false;
			end;
		end;
	end;
	if self.isEntered then		
		if self.flashLeft then
			if self.delayOnLeft >= -1 then
				self.delayOnLeft = self.delayOnLeft -dt;
			end;
			if not self.flashlightsLeftActive then
				if self.delayOnLeft < 0 then
					self.delayOffLeft = 500;
					self.flashlightsLeftActive = true;
					playSample(self.turnSound, 1, 0.7, 0);
					if self.attachedTrailer then
						self.flashlightsLeft2Active = true;
					end;
				end;
			end;
			if self.delayOffLeft >= 0 then
				self.delayOffLeft = self.delayOffLeft -dt;
			end;
			if self.flashlightsLeftActive then
				if self.delayOffLeft < 0 then
					self.delayOnLeft = 500;
					self.flashlightsLeftActive = false;
					playSample(self.turnSound2, 1, 0.7, 0);
					if self.attachedTrailer then
						self.flashlightsLeft2Active = false;
					end;
				end;
			end;
			self.rotationMaxBlinkerLinks = true;
			self.rotationMaxBlinkerRechts = false;
		else
			self.rotationMaxBlinkerLinks = false;
		end;
	end;	
	
	if self.isEntered then
		if self.fuelWarner then
			if self.delayFuelOn >= 0 then
				self.delayFuelOn = self.delayFuelOn -dt;
			end;
			if not self.fuelWarnlightsActive then
				if self.delayFuelOn < 0 then
					self.delayFuelOff = 500;
					self.fuelWarnlightsActive = true;
				end;
			end;
			if self.delayFuelOff >= 0 then
				self.delayFuelOff = self.delayFuelOff -dt;
			end;
			if self.fuelWarnlightsActive then
				if self.delayFuelOff < 0 then
					self.delayFuelOn = 500;
					self.fuelWarnlightsActive = false;
				end;
			end;
			if self.delayFuelSoundOn >= 0 then
				self.delayFuelSoundOn = self.delayFuelSoundOn -dt;
			end;
			if not self.fuelWarnSoundRunning then
				if self.delayFuelSoundOn < 0 then
					self.delayFuelSoundOff = 4000;
					playSample(self.fuelWarnSound, 1, 1, 0);
					self.fuelWarnSoundRunning = true;
				end;
			end;
			if self.delayFuelSoundOff >= 0 then
				self.delayFuelSoundOff = self.delayFuelSoundOff -dt;
			end;
			if self.fuelWarnSoundRunning then
				if self.delayFuelSoundOff < 0 then
					self.delayFuelSoundOn = 20000;
					stopSample(self.fuelWarnSound);
					self.fuelWarnSoundRunning = false;
				end;
			end;	
		else
			self.fuelWarnlightsActive = false;
			self.delayFuelOn = 10;
			self.delayFuelSoundOn = 10;
			stopSample(self.fuelWarnSound);
			self.fuelWarnSoundRunning = false;
		end;
		local fuelWarn = 75;
		if self.fuelFillLevel < fuelWarn then
			self.fuelWarner = true;
		else
			self.fuelWarner = false;
		end;
	end;	
	
	if self.isEntered then
	
		if g_currentMission.environment.lastRainScale <= 0.1 and g_currentMission.environment.timeSinceLastRain > 30 then
			self.rainWarning = false;
		else
			self.rainWarning = true;
		end;
		
		if self.rainWarning then
			if self.delayWischer1 >= 0 then
				self.delayWischer1 = self.delayWischer1 -dt;
			end;
			if self.delayWischer2 >= 0 then
				self.delayWischer2 = self.delayWischer2 -dt;
			end;
			if not self.rotationMaxWischer then
				if self.delayWischer1 < 0 then
					self.delayWischer2 = 1000;
					self.rotationMaxWischer = true;
					self.rotationMaxWischerblatt = true;
				end;
			end;
			if self.rotationMaxWischer then
				if self.delayWischer2 < 0 then
					self.delayWischer1 = 1000;
					self.rotationMaxWischer = false;
					self.rotationMaxWischerblatt = false;
				end;
			end;
		else
			self.rotationMaxWischer = false;
			self.rotationMaxWischerblatt = false;
		end;
	end;
			
	
	if self.flashRight then
		if self.delayOnRight >= 0 then
			self.delayOnRight = self.delayOnRight -dt;
		end;
		if not self.flashlightsRightActive then
			if self.delayOnRight < 0 then
				self.delayOffRight = 500;
				self.flashlightsRightActive = true;
				playSample(self.turnSound, 1, 0.7, 0);
				if self.attachedTrailer then
					self.flashlightsRight2Active = true;
				end;
			end;
		end;
		if self.delayOffRight >= 0 then
			self.delayOffRight = self.delayOffRight -dt;
		end;
		if self.flashlightsRightActive then
			if self.delayOffRight < 0 then
				self.delayOnRight = 500;
				self.flashlightsRightActive = false;
				playSample(self.turnSound2, 1, 0.7, 0);
				if self.attachedTrailer then
					self.flashlightsRight2Active = false;
				end;
			end;
		end;
		self.rotationMaxBlinkerRechts = true;
		self.rotationMaxBlinkerLinks = false;
	else
		self.rotationMaxBlinkerRechts = false;
	end;
	
	if self.flash then
		if self.delayOn >= 0 then
			self.delayOn = self.delayOn -dt;
		end;
		if not self.flashlightsRightActive then
			if self.delayOn < 0 then
				self.delayOff = 500;
				self.flashlightsRightActive = true;
				self.flashlightsLeftActive = true;
				self.flashlightsWarnerActive = true;
				playSample(self.turnSound, 1, 0.7, 0);
				if self.attachedTrailer then
					self.flashlightsRight2Active = true;
					self.flashlightsLeft2Active = true;
				end;
			end;
		end;
		if self.delayOff >= 0 then
			self.delayOff = self.delayOff -dt;
		end;
		if self.flashlightsRightActive then
			if self.delayOff < 0 then
				self.delayOn = 500;
				self.flashlightsLeftActive = false;
				self.flashlightsRightActive = false;
				self.flashlightsWarnerActive = false;
				playSample(self.turnSound2, 1, 0.7, 0);
				if self.attachedTrailer then
					self.flashlightsRight2Active = false;
					self.flashlightsLeft2Active = false;
				end;
			end;
		end;
		self.rotationMaxBlinkerLinks = false;
		self.rotationMaxBlinkerRechts = false;
	end;
	
	if not self.flashLeft and not self.flashRight then
		if not self.flash then
			self.flashlightsRightActive = false;
			self.flashlightsLeftActive = false;
			if self.attachedTrailer then
				self.flashlightsRight2Active = false;
				self.flashlightsLeft2Active = false;
			end;
		end;
	end;
		
	if not self.flash and not self.flashLeft then
		if not self.flashRight then
			self.flashlightsRightActive = false;
			if self.attachedTrailer then
				self.flashlightsRight2Active = false;
			end;
		end;
	end;
	if not self.flash and not self.flashRight then
		if not self.flashLeft then
			self.flashlightLeftActive = false;
			if self.attachedTrailer then
				self.flashlightLeft2Active = false;
			end;
		end;
	end;
	
	if not self.flash then
		self.flashlightsWarnerActive = false;
	end;
	
	if not self.attachedTrailer then
		self.flashlightsRight2Active = false;
		self.flashlightsLeft2Active = false;
	end;
	
	if self.fuelWarnlightsActive then
		for i=1, self.numFuelWarnlights do
			local fuelWarnlight = self.fuelWarnlights[i];
			setVisibility(fuelWarnlight, self.fuelWarnlightsActive);
		end;
		setVisibility(self.fuelWarningDisplay, self.fuelWarnlightsActive);
	else
		for i=1, self.numFuelWarnlights	do
			local fuelWarnlight = self.fuelWarnlights[i];
			setVisibility(fuelWarnlight, self.fuelWarnlightsActive, false);
		end;
		setVisibility(self.fuelWarningDisplay, self.fuelWarnlightsActive, false);
	end;
	
	if self.brakeWarnlightsActive then
		for i=1, self.numBrakeWarnlights do
			local brakeWarnlight = self.brakeWarnlights[i];
			setVisibility(brakeWarnlight, self.brakeWarnlightsActive);
		end;
		setVisibility(self.handbrakeDisplay, self.brakeWarnlightsActive);
	else
		for i=1, self.numBrakeWarnlights do
			local brakeWarnlight = self.brakeWarnlights[i];
			setVisibility(brakeWarnlight, self.brakeWarnlightsActive, false);
		end;
		setVisibility(self.handbrakeDisplay, self.brakeWarnlightsActive, false);
	end;
	if self.flashlightsLeftActive then
		for i=1, self.numFlashlightsLeft do
			local flashlightLeft = self.flashlightsLeft[i];
			setVisibility(flashlightLeft, self.flashlightsLeftActive);
		end;
	else
		for i=1, self.numFlashlightsLeft do
			local flashlightLeft = self.flashlightsLeft[i];
			setVisibility(flashlightLeft, self.flashlightsLeftActive, false);
		end;
	end;
	
	if self.flashlightsRightActive then
		for i=1, self.numFlashlightsRight do
			local flashlightRight = self.flashlightsRight[i];
			setVisibility(flashlightRight, self.flashlightsRightActive);
		end;
	else
		for i=1, self.numFlashlightsRight do
			local flashlightRight = self.flashlightsRight[i];
			setVisibility(flashlightRight, self.flashlightsRightActive, false);
		end;
	end;
	
	if self.flashlightsLeft2Active then
		for i=1, self.numFlashlightsLeft2 do
			local flashlightLeft2 = self.flashlightsLeft2[i];
			setVisibility(flashlightLeft2, self.flashlightsLeft2Active);
		end;
	else
		for i=1, self.numFlashlightsLeft2 do
			local flashlightLeft2 = self.flashlightsLeft2[i];
			setVisibility(flashlightLeft2, self.flashlightsLeft2Active, false);
		end;
	end;
	
	if self.flashlightsRight2Active then
		for i=1, self.numFlashlightsRight2 do
			local flashlightRight2 = self.flashlightsRight2[i];
			setVisibility(flashlightRight2, self.flashlightsRight2Active);
		end;
	else
		for i=1, self.numFlashlightsRight2 do
			local flashlightRight2 = self.flashlightsRight2[i];
			setVisibility(flashlightRight2, self.flashlightsRight2Active, false);
		end;
	end;
	
	if self.flashlightsWarnerActive then
		for i=1, self.numFlashlightsWarner do
			local flashlightWarner = self.flashlightsWarner[i];
			setVisibility(flashlightWarner, self.flashlightsWarnerActive);
		end;
	else
		for i=1, self.numFlashlightsWarner do
			local flashlightWarner = self.flashlightsWarner[i];
			setVisibility(flashlightWarner, self.flashlightsWarnerActive, false);
		end;
	end;
	
	if self.drucklightsActive then
		for i=1, self.numDrucklights do
			local drucklight = self.drucklights[i];
			setVisibility(drucklight, self.drucklightsActive);
		end;
	else
		for i=1, self.numDrucklights do
			local drucklight = self.drucklights[i];
			setVisibility(drucklight, self.drucklightsActive, false);
		end;
	end;
	
	if self.druckWarnlightsActive then
		for i=1, self.numDruckWarnlights do
			local druckWarnlight = self.druckWarnlights[i];
			setVisibility(druckWarnlight, self.druckWarnlightsActive);
		end;
		setVisibility(self.barDisplay, self.druckWarnlightsActive);
	else
		for i=1, self.numDruckWarnlights do
			local druckWarnlight = self.druckWarnlights[i];
			setVisibility(druckWarnlight, self.druckWarnlightsActive, false);
		end;
		setVisibility(self.barDisplay, self.druckWarnlightsActive, false);
	end;

	if self.isEntered then
        if self.rundumleuchtenAnz > 0 then
          if self.rundumleuchtenAn then
            for i=1, self.rundumleuchtenAnz do
              rotate(self.rundumleuchten[i].rotNode, 0, dt*self.rundumleuchten[i].speed, 0);
            end;
          end;
        end;
      end;

--Twin wheels etc begin
	if self.twinWheelsActive then 
		for i=1, self.numTwinWheels do
			local twinWheel = self.twinWheels[i];
			setVisibility(twinWheel, self.twinWheelsActive);
		end;
	else
		for i=1, self.numTwinWheels do
			local twinWheel = self.twinWheels[i];
			setVisibility(twinWheel, self.twinWheelsActive, false);
		end;
	end;
--Twin wheels etc ende

--Plow wheels etc begin
	if self.plowWheelsActive then
		for i=1, self.numPlowWheels do
			local plowWheel = self.plowWheels[i];
			setVisibility(plowWheel, self.plowWheelsActive);
		end;
	else
	   if not self.twinWheelsActive then
		for i=1, self.numPlowWheels do
			local plowWheel = self.plowWheels[i];
			setVisibility(plowWheel, self.plowWheelsActive, false);
		end;
	   end;
	end;
--Plow wheels etc ende




--Blinkleuchten etc ende
	if self.isEntered then
		if getInputAxis(Input.AXIS_Y) == -1 then
			self.rotationMaxPedal = true;
		else
			self.rotationMaxPedal = false;
		end;
		if getInputAxis(Input.AXIS_Y) == 1 then
			self.rotationMaxBrake = true;
		else
			self.rotationMaxBrake = false;
		end;
		if self.gang0 then
			self.rotationMaxClutch = true;
		else
			self.rotationMaxClutch = false;
		end;
	end;
--Animationen Beginn
	local doTranslate = self.translationMaxAttacher or self.translationMinAttacher
		if self.translationPartAttacher ~= nil and doTranslate then
			local x, y, z = getTranslation(self.translationPartAttacher.node);
			local trans = {x,y,z};
			local newTrans = Utils.getMovedLimitedValues(trans, self.translationPartAttacher.maxTrans, self.translationPartAttacher.minTrans, 3, self.translationPartAttacher.transTime, dt, not self.translationMaxAttacher);
			setTranslation(self.translationPartAttacher.node, unpack(newTrans));
		end;
	
	
	if self.translationPart ~= nil then
		local x, y, z = getTranslation(self.translationPart.node);
        local trans = {x,y,z};
        local newTrans = Utils.getMovedLimitedValues(trans, self.translationPart.maxTrans, self.translationPart.minTrans, 3, self.translationPart.transTime, dt, not self.translationMax);
        setTranslation(self.translationPart.node, unpack(newTrans));
	end;

	if self.rotationPartAttacherPart ~= nil then
		local x, y, z = getRotation(self.rotationPartAttacherPart.node);
		local rot = {x,y,z};
		local newRot = Utils.getMovedLimitedValues(rot, self.rotationPartAttacherPart.maxRot, self.rotationPartAttacherPart.minRot, 3, self.rotationPartAttacherPart.rotTime, dt, not self.rotationMaxAttacherPart);
		setRotation(self.rotationPartAttacherPart.node, unpack(newRot));
	end;
	
	if self.rotationPartWischer ~= nil then
		local x, y, z = getRotation(self.rotationPartWischer.node);
		local rot = {x,y,z};
		local newRot = Utils.getMovedLimitedValues(rot, self.rotationPartWischer.maxRot, self.rotationPartWischer.minRot, 3, self.rotationPartWischer.rotTime, dt, not self.rotationMaxWischer);
		setRotation(self.rotationPartWischer.node, unpack(newRot));
	end;
	
	if self.rotationPartBlinkerRechts ~= nil then
		local x, y, z = getRotation(self.rotationPartBlinkerRechts.node);
		local rot = {x,y,z};
		local newRot = Utils.getMovedLimitedValues(rot, self.rotationPartBlinkerRechts.maxRot, self.rotationPartBlinkerRechts.minRot, 3, self.rotationPartBlinkerRechts.rotTime, dt, not self.rotationMaxBlinkerRechts);
		setRotation(self.rotationPartBlinkerRechts.node, unpack(newRot));
	end;
	
	if self.rotationPartBlinkerLinks ~= nil then
		local x, y, z = getRotation(self.rotationPartBlinkerLinks.node);
		local rot = {x,y,z};
		local newRot = Utils.getMovedLimitedValues(rot, self.rotationPartBlinkerLinks.maxRot, self.rotationPartBlinkerLinks.minRot, 3, self.rotationPartBlinkerLinks.rotTime, dt, not self.rotationMaxBlinkerLinks);
		setRotation(self.rotationPartBlinkerLinks.node, unpack(newRot));
	end;
	
	if self.rotationPartJoystick1 ~= nil then
		local x, y, z = getRotation(self.rotationPartJoystick1.node);
		local rot = {x,y,z};
		local newRot = Utils.getMovedLimitedValues(rot, self.rotationPartJoystick1.maxRot, self.rotationPartJoystick1.minRot, 3, self.rotationPartJoystick1.rotTime, dt, not self.rotationMaxJoystick1);
		setRotation(self.rotationPartJoystick1.node, unpack(newRot));
	end;
	
	if self.rotationPartJoystick2 ~= nil then
		local x, y, z = getRotation(self.rotationPartJoystick2.node);
		local rot = {x,y,z};
		local newRot = Utils.getMovedLimitedValues(rot, self.rotationPartJoystick2.maxRot, self.rotationPartJoystick2.minRot, 3, self.rotationPartJoystick2.rotTime, dt, not self.rotationMaxJoystick2);
		setRotation(self.rotationPartJoystick2.node, unpack(newRot));
	end;
	
	if self.rotationPartJoystick3 ~= nil then
		local x, y, z = getRotation(self.rotationPartJoystick3.node);
		local rot = {x,y,z};
		local newRot = Utils.getMovedLimitedValues(rot, self.rotationPartJoystick3.maxRot, self.rotationPartJoystick3.minRot, 3, self.rotationPartJoystick3.rotTime, dt, not self.rotationMaxJoystick3);
		setRotation(self.rotationPartJoystick3.node, unpack(newRot));
	end;
	
	if self.rotationPartJoystick4 ~= nil then
		local x, y, z = getRotation(self.rotationPartJoystick4.node);
		local rot = {x,y,z};
		local newRot = Utils.getMovedLimitedValues(rot, self.rotationPartJoystick4.maxRot, self.rotationPartJoystick4.minRot, 3, self.rotationPartJoystick4.rotTime, dt, not self.rotationMaxJoystick4);
		setRotation(self.rotationPartJoystick4.node, unpack(newRot));
	end;
	
	local doTranslate = self.translationMaxZylinderHub2 or self.translationMinZylinderHub2
	if self.translationPartZylinderHub2 ~= nil and doTranslate then
		local x, y, z = getTranslation(self.translationPartZylinderHub2.node);
		local trans = {x,y,z};
		local newTrans = Utils.getMovedLimitedValues(trans, self.translationPartZylinderHub2.maxTrans, self.translationPartZylinderHub2.minTrans, 3, self.translationPartZylinderHub2.transTime, dt, not self.translationMaxZylinderHub2);
		setTranslation(self.translationPartZylinderHub2.node, unpack(newTrans));
	end;
	
	local doRotate = self.rotationMaxZylinderHub or self.rotationMinZylinderHub
	if self.rotationPartZylinderHub ~= nil and doRotate then
		local x, y, z = getRotation(self.rotationPartZylinderHub.node);
		local rot = {x,y,z};
		local newRot = Utils.getMovedLimitedValues(rot, self.rotationPartZylinderHub.maxRot, self.rotationPartZylinderHub.minRot, 3, self.rotationPartZylinderHub.rotTime, dt, not self.rotationMaxZylinderHub);
		setRotation(self.rotationPartZylinderHub.node, unpack(newRot));
	end;
	
	local doTranslate = self.translationMaxZylinderKipp2 or self.translationMinZylinderKipp2
	if self.translationPartZylinderKipp2 ~= nil and doTranslate then
		local x, y, z = getTranslation(self.translationPartZylinderKipp2.node);
		local trans = {x,y,z};
		local newTrans = Utils.getMovedLimitedValues(trans, self.translationPartZylinderKipp2.maxTrans, self.translationPartZylinderKipp2.minTrans, 3, self.translationPartZylinderKipp2.transTime, dt, not self.translationMaxZylinderKipp2);
		setTranslation(self.translationPartZylinderKipp2.node, unpack(newTrans));
	end;
	
	local doRotate = self.rotationMaxZylinderKipp or self.rotationMinZylinderKipp
	if self.rotationPartZylinderKipp ~= nil and doRotate then
		local x, y, z = getRotation(self.rotationPartZylinderKipp.node);
		local rot = {x,y,z};
		local newRot = Utils.getMovedLimitedValues(rot, self.rotationPartZylinderKipp.maxRot, self.rotationPartZylinderKipp.minRot, 3, self.rotationPartZylinderKipp.rotTime, dt, not self.rotationMaxZylinderKipp);
		setRotation(self.rotationPartZylinderKipp.node, unpack(newRot));
	end;
	
	local doRotate = self.rotationMaxGelenk1 or self.rotationMinGelenk1
	if self.rotationPartGelenk1 ~= nil and doRotate then
		local x, y, z = getRotation(self.rotationPartGelenk1.node);
		local rot = {x,y,z};
		local newRot = Utils.getMovedLimitedValues(rot, self.rotationPartGelenk1.maxRot, self.rotationPartGelenk1.minRot, 3, self.rotationPartGelenk1.rotTime, dt, not self.rotationMaxGelenk1);
		setRotation(self.rotationPartGelenk1.node, unpack(newRot));
	end;
	
	local doRotate = self.rotationMaxGelenk2 or self.rotationMinGelenk2
	if self.rotationPartGelenk2 ~= nil and doRotate then
		local x, y, z = getRotation(self.rotationPartGelenk2.node);
		local rot = {x,y,z};
		local newRot = Utils.getMovedLimitedValues(rot, self.rotationPartGelenk2.maxRot, self.rotationPartGelenk2.minRot, 3, self.rotationPartGelenk2.rotTime, dt, not self.rotationMaxGelenk2);
		setRotation(self.rotationPartGelenk2.node, unpack(newRot));
	end;
	
	
	local maxDrehzahl = 2200;
	if self.rotationPartDrehzahl1 ~= nil then
		local x, y, z = getRotation(self.rotationPartDrehzahl1.node);
		y = ((self.rotationPartDrehzahl1.maxRot[2] - self.rotationPartDrehzahl1.minRot[2]) / maxDrehzahl) * self.motor.lastMotorRpm + self.rotationPartDrehzahl1.minRot[2];
		setRotation(self.rotationPartDrehzahl1.node, x, y ,z);
	end;
	if self.rotationPartDrehzahl2 ~= nil then
		local x, y, z = getRotation(self.rotationPartDrehzahl2.node);
		y = ((self.rotationPartDrehzahl2.maxRot[2] - self.rotationPartDrehzahl2.minRot[2]) / maxDrehzahl) * self.motor.lastMotorRpm + self.rotationPartDrehzahl2.minRot[2];
		setRotation(self.rotationPartDrehzahl2.node, x, y ,z);
	end;
	
	local maxKmh = 55;
	if self.rotationPartKmh1 ~= nil then
		local x, y, z = getRotation(self.rotationPartKmh1.node);
		y = ((self.rotationPartKmh1.maxRot[2] - self.rotationPartKmh1.minRot[2]) / maxKmh) * self.lastSpeed*3600 + self.rotationPartKmh1.minRot[2];
		setRotation(self.rotationPartKmh1.node, x, y ,z);
	end;
	if self.rotationPartKmh2 ~= nil then
		local x, y, z = getRotation(self.rotationPartKmh2.node);
		y = ((self.rotationPartKmh2.maxRot[2] - self.rotationPartKmh2.minRot[2]) / maxKmh) * self.lastSpeed*3600 + self.rotationPartKmh2.minRot[2];
		setRotation(self.rotationPartKmh2.node, x, y ,z);
	end;
		
	
	if self.rotationPartTanknadel ~= nil then
		local x, y, z = getRotation(self.rotationPartTanknadel.node);
		y = ((self.rotationPartTanknadel.maxRot[2] - self.rotationPartTanknadel.minRot[2]) / self.fuelCapacity) * self.fuelFillLevel + self.rotationPartTanknadel.minRot[2];
		setRotation(self.rotationPartTanknadel.node, x, y ,z);
	end;
	
	if self.rotationPartTanknadel2 ~= nil then
		local x, y, z = getRotation(self.rotationPartTanknadel2.node);
		y = ((self.rotationPartTanknadel2.maxRot[2] - self.rotationPartTanknadel2.minRot[2]) / self.fuelCapacity) * self.fuelFillLevel + self.rotationPartTanknadel2.minRot[2];
		setRotation(self.rotationPartTanknadel2.node, x, y ,z);
	end;
	

	local maxDruck = 10000;
	if self.rotationPartDruck ~= nil then
		local x, y, z = getRotation(self.rotationPartDruck.node);
		y = ((self.rotationPartDruck.maxRot[2] - self.rotationPartDruck.minRot[2]) / maxDruck) * self.druckLevel + self.rotationPartDruck.minRot[2];
		setRotation(self.rotationPartDruck.node, x, y ,z);
	end;
	
	local maxDruckLight = 10000;
	if self.rotationPartDruckLight ~= nil then
		local x, y, z = getRotation(self.rotationPartDruckLight.node);
		y = ((self.rotationPartDruckLight.maxRot[2] - self.rotationPartDruckLight.minRot[2]) / maxDruckLight) * self.druckLevel + self.rotationPartDruckLight.minRot[2];
		setRotation(self.rotationPartDruckLight.node, x, y ,z);
	end;
	
	if self.rotationPartPedal ~= nil then
		local x, y, z = getRotation(self.rotationPartPedal.node);
		local rot = {x,y,z};
		local newRot = Utils.getMovedLimitedValues(rot, self.rotationPartPedal.maxRot, self.rotationPartPedal.minRot, 3, self.rotationPartPedal.rotTime, dt, not self.rotationMaxPedal);
		setRotation(self.rotationPartPedal.node, unpack(newRot));
	end;
	
	if self.rotationPartBrake ~= nil then
		local x, y, z = getRotation(self.rotationPartBrake.node);
		local rot = {x,y,z};
		local newRot = Utils.getMovedLimitedValues(rot, self.rotationPartBrake.maxRot, self.rotationPartBrake.minRot, 3, self.rotationPartBrake.rotTime, dt, not self.rotationMaxBrake);
		setRotation(self.rotationPartBrake.node, unpack(newRot));
	end;
	
	if self.rotationPartClutch ~= nil then
		local x, y, z = getRotation(self.rotationPartClutch.node);
		local rot = {x,y,z};
		local newRot = Utils.getMovedLimitedValues(rot, self.rotationPartClutch.maxRot, self.rotationPartClutch.minRot, 3, self.rotationPartClutch.rotTime, dt, not self.rotationMaxClutch);
		setRotation(self.rotationPartClutch.node, unpack(newRot));
	end;
	if self.isEntered then
		if self.brakelightsActive then
			for i=1, self.numBrakelights do
				local brakelight = self.brakelights[i];
				setVisibility(brakelight, self.brakelightsActive);
			end;
		else
			for i=1, self.numBrakelights do
				local brakelight = self.brakelights[i];
				setVisibility(brakelight, self.brakelightsActive, false);
			end;
		end;
	end;
		
	
	if self.rotationPartWischerblatt ~= nil then
		local x, y, z = getRotation(self.rotationPartWischerblatt.node);
		local rot = {x,y,z};
		local newRot = Utils.getMovedLimitedValues(rot, self.rotationPartWischerblatt.maxRot, self.rotationPartWischerblatt.minRot, 3, self.rotationPartWischerblatt.rotTime, dt, not self.rotationMaxWischerblatt);
		setRotation(self.rotationPartWischerblatt.node, unpack(newRot));
	end;
	
	if self.isEntered then
		if self.worklightsActive then
			for i=1, self.numWorklights do
				local worklight = self.worklights[i];
				setVisibility(worklight, self.worklightsActive);
			end;
			setVisibility(self.workLightDisplay, self.worklightsActive); 
		else
			for i=1, self.numWorklights do
				local worklight = self.worklights[i];
				setVisibility(worklight, self.worklightsActive, false);
			end;
			setVisibility(self.workLightDisplay, false);
		end;
	end;

	--RPM Displays
	if self.isEntered then
		if self.farlightsActive or self.frontlightsActive then
			for i=1, self.numrpmlightdisplays do
          			if self.motor.lastMotorRpm >= self.rpmlightdisplays[i].rpmMinValue and self.motor.lastMotorRpm < self.rpmlightdisplays[i].rpmMaxValue then 
        				setVisibility(self.rpmlightdisplays[i].rotNode, true);
	    			else
					setVisibility(self.rpmlightdisplays[i].rotNode, false); 
	    			end;
      		end;
			for i=1, self.numrpmdisplays do
				setVisibility(self.rpmdisplays[i].rotNode, false); 
      		end;
		else
			for i=1, self.numrpmdisplays do
          			if self.motor.lastMotorRpm >= self.rpmdisplays[i].rpmMinValue and self.motor.lastMotorRpm < self.rpmdisplays[i].rpmMaxValue then 
        				setVisibility(self.rpmdisplays[i].rotNode, true);
	    			else
					setVisibility(self.rpmdisplays[i].rotNode, false); 
	    			end;
      		end;
			for i=1, self.numrpmlightdisplays do
				setVisibility(self.rpmlightdisplays[i].rotNode, false); 
      		end;
		end;
	end;


	
	if self.attachedTrailer == nil and self.trailerAttacherJoint2 ~= nil then
	    local px1, py1, pz1 = getWorldTranslation(self.trailerAttacherJoint1);
		local px2, py2, pz2 = getWorldTranslation(self.trailerAttacherJoint2);
        local nearestTrailer = nil
		local nearestDistance = 0.4;
		for i=1, table.getn(g_currentMission.trailers) do
			local vx, vy, vz = getWorldTranslation(g_currentMission.trailers[i].attacherJoint);
	        local distance1 = Utils.vector2Length(px1-vx, pz1-vz);
			local distance2 = Utils.vector2Length(px2-vx, pz2-vz);
			if distance1 < nearestDistance then
				self.trailerAttacherJoint = self.trailerAttacherJoint1;
	        end;
            if distance2 < nearestDistance then
	            self.trailerAttacherJoint = self.trailerAttacherJoint2;
			end;
        end;
	end;
	if self.isEntered then
		local brakeTorque = 0.00001;
		if getInputAxis(Input.AXIS_Y) == 1 then
			self.brakelightsActive = true;
			self.bremzleuchtenAn = true;
			if self.druckLevel >= 5500 then
				brakeTorque = 200;
				self.motor.brakeForce = brakeTorque;
				if not self.brakeSoundRunning then
					playSample(self.brakeSound, 1, 1, 0);
					self.brakeSoundRunning = true;
				end;
				self.brakeReleaseSoundRunning = false;
			end;
			if not self.braking then
				if self.druckLevel > 0 then
					if self.attachedTrailer then
						self.druckLevel = self.druckLevel -350;
					else
						self.druckLevel = self.druckLevel -250;
					end;
				end;
				self.braking = true;
			end;
		else
			if not self.handbrake then
				brakeTorque = 0.00001;
				self.motor.brakeForce = brakeTorque;
			end;
			self.brakelightsActive = false;
			self.bremzleuchtenAn = false;
			self.braking = false;
			if self.druckLevel >= 5500 then
				if not self.brakeReleaseSoundRunning then
					playSample(self.brakeReleaseSound, 1, 1, 0);
					self.brakeReleaseSoundRunning = true;
				end;
				self.brakeSoundRunning = false;
			end;
		end;
	
		if self.handbrake then
			brakeTorque = 250;
			self.motor.brakeForce = brakeTorque;
			self.brakeWarnlightsActive = true;
			self.handbrakeSoundRunning = false;
		else
			self.brakeWarnlightsActive = false;
			if not self.handbrakeSoundRunning then
				playSample(self.handbrakeSound, 1, 1, 0);
				self.handbrakeSoundRunning = true;
			end;
		end;
	end;
	
	if self.frontLoaderAttachedObject ~= nil then
        self.frontLoaderAttachedObject:update(dt);
    end;


    local jointDesc = self.frontLoaderJointDesc;
    local jointFrameInvalid = false;
    if jointDesc.rotationNode ~= nil then
        if jointDesc.moveDirection ~= 0 then
            local x, y, z = getRotation(jointDesc.rotationNode);
            local rot = {x,y,z};
            local newRot = Utils.getMovedLimitedValues(rot, jointDesc.maxRot, jointDesc.minRot, 3, jointDesc.moveTime, dt, jointDesc.moveDirection == 1);
            setRotation(jointDesc.rotationNode, unpack(newRot));
            for i=1, 3 do
                if math.abs(newRot[i] - rot[i]) > 0.001 then
                    jointFrameInvalid = true;
                end;
            end;
        end;
    end;
    if jointDesc.rotationNode2 ~= nil then
        if jointDesc.moveDirection2 ~= 0 then
            local x, y, z = getRotation(jointDesc.rotationNode2);
            local rot = {x,y,z};
            local newRot = Utils.getMovedLimitedValues(rot, jointDesc.maxRot2, jointDesc.minRot2, 3, jointDesc.moveTime2, dt, jointDesc.moveDirection2 == 1);
            setRotation(jointDesc.rotationNode2, unpack(newRot));
            for i=1, 3 do
                if math.abs(newRot[i] - rot[i]) > 0.001 then
                    jointFrameInvalid = true;
                end;
            end;
        end;
    end;

    if jointFrameInvalid and jointDesc.jointIndex ~= 0 then
        setJointFrame(jointDesc.jointIndex, 0, jointDesc.jointTransform);
    end;
	
    self.firstTimeRun = true;




--AP start

  if self.fL ~= nil and self.villogo then
     rotate(self.fL.node,0,-0.5,0);
  end; 
  

        if self.autoPilotPresent then
            if self.autoPilotEnabled then
                self.autoRotateBackSpeed = 0;
            else
                self.autoRotateBackSpeed = self.autoRotateBackSpeedBackup;
            end;
        end;

    if self.stoppedap then
	g_currentMission:addExtraPrintText("Varakoz");
	if self.isEntered and self.motor.speedLevel ~= 0 then
		self.stoppedap=false;
	end;
    end;	
     
        
            if self.autoPilotEnabled and self.movingDirection > 0  and self.autoPilotAreaLeft.available then
                self.turnDirection = 0;

                

                if self.autoPilotAreaRight.available and self.autoPilotAreaRight.active then
                    local x,y,z = getWorldTranslation(self.autoPilotAreaRight.startOutside);
                    local x1,y1,z1 = getWorldTranslation(self.autoPilotAreaRight.widthOutside);
                    local x2,y2,z2 = getWorldTranslation(self.autoPilotAreaRight.heightOutside);
                    local right = Utils.getDensity(g_currentMission.cuttedWheatId, 0, x, z, x1, z1, x2, z2);

                    local x,y,z = getWorldTranslation(self.autoPilotAreaRight.startInside);
                    local x1,y1,z1 = getWorldTranslation(self.autoPilotAreaRight.widthInside);
                    local x2,y2,z2 = getWorldTranslation(self.autoPilotAreaRight.heightInside);
                    local left = Utils.getDensity(g_currentMission.cuttedWheatId, 0, x, z, x1, z1, x2, z2);

                    self.turnDirection = right -(9 -left);

                    if (left < 1 and right < 1) or (left > 8 and right > 8) then
                        self.autoPilotDelayRight = self.autoPilotDelayRight +1;
                    else
                        self.autoPilotDelayRight = 0;
                    end;
                end;

                if self.autoPilotAreaLeft.available and self.autoPilotAreaLeft.active then
                    local x,y,z = getWorldTranslation(self.autoPilotAreaLeft.startOutside);
                    local x1,y1,z1 = getWorldTranslation(self.autoPilotAreaLeft.widthOutside);
                    local x2,y2,z2 = getWorldTranslation(self.autoPilotAreaLeft.heightOutside);
                    local left = Utils.getDensity(g_currentMission.cuttedWheatId, 0, x, z, x1, z1, x2, z2);

                    local x,y,z = getWorldTranslation(self.autoPilotAreaLeft.startInside);
                    local x1,y1,z1 = getWorldTranslation(self.autoPilotAreaLeft.widthInside);
                    local x2,y2,z2 = getWorldTranslation(self.autoPilotAreaLeft.heightInside);
                    local right = Utils.getDensity(g_currentMission.cuttedWheatId, 0, x, z, x1, z1, x2, z2);

                    self.turnDirection = (9 -right) -left;
                    
                    if (left < 1 and right < 1) or (left > 8 and right > 8) then
                        self.autoPilotDelayLeft = self.autoPilotDelayLeft +1;
                    else
                        self.autoPilotDelayLeft = 0;
                    end;
                end;

                local acceleration = 0;
                if g_currentMission.allowSteerableMoving and not self.playMotorSound then
                    if self.motor.speedLevel ~= 0 then 
                        acceleration = 1.0;
                    end;
                end;
                if self.fuelFillLevel == 0 then
                    acceleration = 0;
                end;

                local rotScale = math.min(1.0/(self.lastSpeed*50+1), 1);
                if self.turnDirection < 0 then
                    self.rotatedTime = math.min(self.rotatedTime - dt/2500*self.turnDirection*rotScale, 0.2);
                elseif self.turnDirection > 0 then
                    self.rotatedTime = math.max(self.rotatedTime - dt/2500*self.turnDirection*rotScale, -0.2);
                else
                    self.rotatedTime = 0;
                end;

                if self.firstTimeRun then
                    WheelsUtil.updateWheels(self, dt, self.lastSpeed, acceleration, false, 0)
                end;
            
                if self.steering ~= nil then
                    setRotation(self.steering, 0, self.rotatedTime*self.steeringSpeed, 0);
                end;

                if self.autoPilotDelayLeft > 5 or self.autoPilotDelayRight > 5 then
                    self.autoRotateBackSpeed = self.autoRotateBackSpeedBackup;
		    if not self.isEntered then

			self.motor.speedLevel = 0;
			self.stoppedap=true;
                    else
                        self.autoPilotEnabled = false;
                        playSample(self.WarningSound, 2, 10.6, 0);
		    end;
                end;
		
            else
                self.autoPilotDelayLeft = 0;
                self.autoPilotDelayRight = 0;

                    self.autoRotateBackSpeed = self.autoRotateBackSpeedBackup;
		    if self.autoPilotEnabled and self.stoppedap then 
                           self.autoPilotEnabled = false;
		    end;                
            end;
--AP ende

end;


function Vehicle4:drawGrainLevel(level, capacity, warnPercent)

    local percent = level/capacity*100;
    setTextBold(true);

    if percent >= warnPercent then
        setTextColor(1.0, 0.0, 0.0, 1.0);
    else
        setTextColor(1.0, 1.0, 1.0, 1.0);
    end;

    renderText(self.hudBasePoxX+0.031, self.hudBasePoxY+0.006, 0.03, string.format("%d(%d%%)", level, percent));
    if percent >= warnPercent then
        setTextColor(1.0, 1.0, 1.0, 1.0);
    end;
    setTextBold(false);
end;

function Vehicle4:draw()

    if self.selectedImplement ~= 0 then
        self.attachedImplements[self.selectedImplement].object:draw();
    end;
    if self.attachedTrailer ~= nil then
        self.attachedTrailer:draw();
        self:drawGrainLevel(self.attachedTrailer.fillLevel, self.attachedTrailer.capacity, 101);
    end;
    if table.getn(self.attachedImplements) > 1 then
        --g_currentMission:addHelpButtonText("Gerteauswahl wechseln", InputBinding.SWITCH_IMPLEMENT);
        g_currentMission:addHelpButtonText(g_i18n:getText("Change_tools"), InputBinding.SWITCH_IMPLEMENT);
    end;

    if self.showWaterWarning then
        --g_currentMission:addWarning("Fahren Sie nicht zu tief ins Wasser", 0.05, 0.025+0.007);
        g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_depth_into_the_water"), 0.05, 0.025+0.007);  
    end;
	
	  
    local attachableInRange = self:frontLoaderAttachableInRange()
	if attachableInRange ~= nil and self.frontLoaderAttachedObject == nil then
		g_currentMission:addExtraPrintText("Taste NUM 0: Ankoppeln");
	end;
	
    local kmh = self.lastSpeed*3600;
    if self.isEntered then

        self.hudBaseOverlay:render();

        --if kmh >= 1 and kmh < 100 then

         --   setTextColor(1.0, 1.0, 1.0, 1.0);

         --   local textSize = 0.075;
         --   local yPos = 0.05;
         --   setTextBold(true);
          --  renderText(0.79, yPos, textSize, string.format("%d", kmh));
         --   renderText(0.86, yPos, textSize, "km/h");
          --  setTextBold(false);
        --end;
        --setTextColor(1.0, 1.0, 1.0, 1.0);
        --renderText(0.77, 0.12, 0.05, "Diesel: " .. string.format("%d l", self.fuelFillLevel));


        setTextBold(true);
        setTextColor(1.0, 1.0, 1.0, 1.0);
        if kmh > 0 and kmh < 100 then
            renderText(self.hudBasePoxX+0.007, self.hudBasePoxY+0.095, 0.06, string.format("%2d", kmh));
        end;
        renderText(self.hudBasePoxX+0.058, self.hudBasePoxY+0.094, 0.06, "km/h");
        renderText(self.hudBasePoxX+0.031, self.hudBasePoxY+0.071, 0.03, string.format("%d", g_currentMission.missionStats.money));
        
        local fuelWarn = 50;
        if self.fuelFillLevel < fuelWarn then
            setTextColor(1,0,0,1);
        end;
        renderText(self.hudBasePoxX+0.031, self.hudBasePoxY+0.039, 0.03, string.format("%d liter", self.fuelFillLevel));
        if self.fuelFillLevel < fuelWarn then
            setTextColor(1,1,1,1);
        end;
        
        
        setTextBold(false);


        if self.hasRefuelStationInRange and not self.doRefuel and self.fuelFillLevel ~= self.fuelCapacity then
            g_currentMission:addHelpButtonText(g_i18n:getText("Refuel"), InputBinding.REFUEL);
        end;
    end;
	
	if self.handbrake then
		setTextBold(true);
		setTextColor(1,0,0,1);
		if getInputAxis(Input.AXIS_Y) == -1 then
			setTextBold(true);
			setTextColor(1,0,0,1);
			renderText(0.3, 0.3, 0.05, "BITTE HANDBREMSE LSEN!");
		end;
		renderText(0.875, 0.19, 0.015, "(!) Handbremse Aktiv");	
	end;
	if self.isEntered and self.hudGearOverlay ~= nil then
		setTextColor(1,1,1,1);
		setTextBold(false);
		self.hudGearOverlay:render();
	end;
	
	if self.isEntered then
		setTextColor(0.0,0.0,0.0,1);
		setTextBold(true);
		renderText(0.883, 0.22, 0.015, "JD Gear Display");
	end;
	if self.isEntered then
		setTextBold(true);
		if not self.isAutomatic then
			if self.gang02 then
				setTextColor(0,1,0,1);
				renderText(0.96, 0.2, 0.02, "R2");
			else
				setTextColor(0.25,0.25,0.25,0.9);
				renderText(0.96, 0.2, 0.02, "R2");
			end;
			if self.gang01 then
				setTextColor(0,1,0,1);
				renderText(0.94, 0.2, 0.02, "R1");
			else
				setTextColor(0.25,0.25,0.25,0.9);
				renderText(0.94, 0.2, 0.02, "R1");
			end;
			if self.gang0 then
				setTextColor(1,0,0,1);
				renderText(0.93, 0.2, 0.02, "N");
			else
				setTextColor(0.25,0.25,0.25,0.9);
				renderText(0.93, 0.2, 0.02, "N");
			end;
			if self.gang1 then
				setTextColor(0,1,0,1);
				renderText(0.92, 0.2, 0.02, "1");
			else
				setTextColor(0.25,0.25,0.25,0.9);
				renderText(0.92, 0.2, 0.02, "1");
			end;
			if self.gang2 then
				setTextColor(0,1,0,1);
				renderText(0.91, 0.2, 0.02, "2");
			else
				setTextColor(0.25,0.25,0.25,0.9);
				renderText(0.91, 0.2, 0.02, "2");
			end;
			if self.gang3 then
				setTextColor(0,1,0,1);
				renderText(0.9, 0.2, 0.02, "3");
			else
				setTextColor(0.25,0.25,0.25,0.9);
				renderText(0.9, 0.2, 0.02, "3");
			end;
			if self.gang4 then
				setTextColor(0,1,0,1);
				renderText(0.89, 0.2, 0.02, "4");
			else
				setTextColor(0.25,0.25,0.25,0.9);
				renderText(0.89, 0.2, 0.02, "4");
			end;
			if self.gang5 then
				setTextColor(0,1,0,1);
				renderText(0.88, 0.2, 0.02, "5");
			else
				setTextColor(0.25,0.25,0.25,0.9);
				renderText(0.88, 0.2, 0.02, "5");
			end;
			if self.gang6 then
				setTextColor(0,1,0,1);
				renderText(0.87, 0.2, 0.02, "6");
			else
				setTextColor(0.25,0.25,0.25,0.9);
				renderText(0.87, 0.2, 0.02, "6");
			end;
		end;
		if self.isAutomatic then
			renderText(0.87, 0.2, 0.02, "Automatik");
			if self.forward then
				setTextColor(0,1,0,1);
				renderText(0.935, 0.2, 0.02, "V");
			else
				setTextColor(0.25,0.25,0.25,0.9);
				renderText(0.935, 0.2, 0.02, "V");
			end;
			if self.neutral then
				setTextColor(1,0,0,1);
				renderText(0.95, 0.2, 0.02, "N");
			else
				setTextColor(0.25,0.25,0.25,0.9);
				renderText(0.95, 0.2, 0.02, "N");
			end;
			if self.reverseGear then
				setTextColor(0,1,0,1);
				renderText(0.965, 0.2, 0.02, "R");
			else
				setTextColor(0.25,0.25,0.25,0.9);
				renderText(0.965, 0.2, 0.02, "R");
			end;
		end;
		if self.druckWarning then
			setTextBold(true);
			setTextColor(1,0,0,1);
			renderText(0.3, 0.6, 0.03, "   BREMSDRUCK ZU NIEDRIG\nKeine Bremskraft vorhanden!");
		end;
	end;

--AP start
   if self.autoPilotPresent then
        if not self.autoPilotEnabled then
            if self.autoPilotAreaLeft.active then
                g_currentMission:addExtraPrintText("Billentyu P; L,R: Autopilot bal kivalasztva");
            elseif self.autoPilotAreaRight.active then
                g_currentMission:addExtraPrintText("Billentyu P; L,R: Autopilot jobb kivalasztva");	

            else
                g_currentMission:addExtraPrintText("Billentyu P; L,R: Autopilot kikapcsolva");
            end;
        else
            if self.autoPilotAreaLeft.active then
                if self.movingDirection > 0 then
                    g_currentMission:addExtraPrintText("Billentyu P; L,R: Autopilot bal (AKTIV) bekapcsolva");
                else
                    g_currentMission:addExtraPrintText("Billentyu P; L,R: Autopilot bal (INAKTIV) bekapcsolva");
                end;
            elseif self.autoPilotAreaRight.active then
                if self.movingDirection > 0  then 
                    g_currentMission:addExtraPrintText("Billentyu P; L,R: Autopilot jobb (AKTIV) bekapcsolva");
                else
                    g_currentMission:addExtraPrintText("Billentyu P; L,R: Autopilot jobb (INAKTIV) bekapcsolva");
                end;
            end;
        end;
    end;

--AP Stop
		
end;

function Vehicle4:onEnter()
    self.isEntered = true;
	
	self.gang0 = true;
	
    local motorSoundOffset = 0;
    if self.motorStartSound ~= nil then
        setSamplePitch(self.motorStartSound, self.motorStartSoundPitchOffset);
        playSample(self.motorStartSound, 1, 1, 0);
        --motorSoundOffset = 800;
        motorSoundOffset = getSampleDuration(self.motorStartSound);
    end;

    self.playMotorSound = true;
    self.playMotorSoundTime = self.time+motorSoundOffset;
    self.reverseDriveSoundEnabled = false;

    self.camIndex = 1;
    self.cameras[self.camIndex]:onActivate();

    for i=1, self.numrpmdisplays do
          if self.motor.lastMotorRpm >= self.rpmdisplays[i].rpmMinValue and self.motor.lastMotorRpm < self.rpmdisplays[i].rpmMaxValue then 
        	setVisibility(self.rpmdisplays[i].rotNode, true);
	    else
		setVisibility(self.rpmdisplays[i].rotNode, false); 
	    end;
    end;

    Utils.setEmittingState(self.exhaustParticleSystems, true)
end;

function Vehicle4:onLeave()
    self.isEntered = false;
    if self.motorSound ~= nil then
        stopSample(self.motorSound);
    end;
    if self.reverseDriveSoundEnabled then
        stopSample(self.reverseDriveSound);
        self.reverseDriveSoundEnabled = false;
    end;
    self.cameras[self.camIndex]:onDeactivate();

    self.doRefuel = false;

    self.rundumleuchtenAn = false;
    for i=1, self.rundumleuchtenAnz do
       setVisibility(self.rundumleuchten[i].light, self.rundumleuchtenAn);
    end;

    setRotation(self.exhaustCover, 0, Utils.degToRad(180), 0);

    self.bremzleuchtenAn = false;
    for i=1, self.bremzleuchtenAnz do
       setVisibility(self.bremzleuchten[i].light, self.bremzleuchtenAn);
    end;

    self.frontleuchtenAn = false;
    for i=1, self.frontleuchtenAnz do
       setVisibility(self.frontleuchten[i].light, self.frontleuchtenAn);
    end;

    self.farleuchtenAn = false;
    for i=1, self.farleuchtenAnz do
       setVisibility(self.farleuchten[i].light, self.farleuchtenAn);
    end;

    self.leuchtenAn = false;
    for i=1, self.leuchtenAnz do
       setVisibility(self.leuchten[i].light, self.leuchtenAn);
    end;

    for i=1, self.numrpmdisplays do
	setVisibility(self.rpmdisplays[i].rotNode, false); 
    end;
    if self.numrpmdisplays > 0 then
	setVisibility(self.rpmdisplays[1].rotNode, true); 
    end;

    for i=1, self.numrpmlightdisplays do
	setVisibility(self.rpmlightdisplays[i].rotNode, false); 
    end;

    Utils.setEmittingState(self.exhaustParticleSystems, false)
    for i=1, self.numLights do
        setVisibility(self.lights[i], false);
    end;
    self.lightsActive = false;
	self.farlightsActive = false;
	self.frontlightsActive = false;
	

    for i=1, table.getn(self.wheels) do
       setWheelShapeProps(self.wheels[i].node, self.wheels[i].wheelShape, 0, self.motor.brakeForce, 0);
    end;
	
	self.worklightsActive = false;
	stopSample(self.compressorSound);
    self.compressorSoundRunning = false;
	self.helpPanel = false;
	Utils.setEmittingState(self.exhaustParticleSystems2, false);
	Utils.setEmittingState(self.exhaustParticleSystems3, false);
	Utils.setEmittingState(self.exhaustParticleSystems4, false);
	Utils.setEmittingState(self.exhaustParticleSystems5, false);
	self.motor.forwardGearRatio = 1;
	self.motor.backwardGearRatio = 1;
	
	self.gang0 = true;
	self.neutral = true;
	self.isAutomatic = false;
	self.gang02 = false;
	self.gang01 = false;
	self.gang1 = false;
	self.gang2 = false;
	self.gang3 = false;
	self.gang4 = false;
	self.gang5 = false;
	self.gang6 = false;
	self.forward = false;
	self.reverseGear = false;
	
end;

function Vehicle4:attachImplement(object, index)

    local jointDesc = self.attacherJoints[index];
    local implement = {};
    implement.object = object;
    implement.object:onAttach(self);
    implement.jointDescIndex = index;

    local constr = JointConstructor:new();
    constr:setActors(self.rootNode, implement.object.rootNode);
    constr:setJointTransforms(jointDesc.jointTransform, implement.object.attacherJoint);
    --constr:setBreakable(20, 10);

    implement.jointRotLimit = {};
    implement.jointTransLimit = {};
    for i=1, 3 do
        local rotLimit = jointDesc.maxRotLimit[i];
        if implement.object.fixedAttachRotation then
            rotLimit = 0;
        end;
        constr:setRotationLimit(i-1, -rotLimit, rotLimit);
        implement.jointRotLimit[i] = rotLimit;

        constr:setTranslationLimit(i-1, true, -jointDesc.maxTransLimit[i], jointDesc.maxTransLimit[i]);
        implement.jointTransLimit[i] = jointDesc.maxTransLimit[i];
    end;
    if jointDesc.rotationNode ~= nil then
        setRotation(jointDesc.rotationNode, unpack(jointDesc.maxRot));
    end;
    if jointDesc.rotationNode2 ~= nil then
        setRotation(jointDesc.rotationNode2, unpack(jointDesc.maxRot2));
    end;
    jointDesc.jointIndex = constr:finalize();
    jointDesc.moveDown = false;
    table.insert(self.attachedImplements, implement);

    if self.selectedImplement == 0 then
        self.selectedImplement = 1;
        implement.object:onSelect();
    end;

end;

function Vehicle4:detachImplement(index)

    local implement = self.attachedImplements[index];
    local jointDesc = self.attacherJoints[implement.jointDescIndex];

    removeJoint(jointDesc.jointIndex);
    jointDesc.jointIndex = 0;

    if index == self.selectedImplement then
        self.attachedImplements[index].object:onDeselect();
    end;
    implement.object:onDetach();
    implement.object = nil;
    if jointDesc.topArm ~= nil then
        setRotation(jointDesc.topArm.rotationNode, jointDesc.topArm.rotX, jointDesc.topArm.rotY, jointDesc.topArm.rotZ);
        if jointDesc.topArm.translationNode ~= nil then
            setTranslation(jointDesc.topArm.translationNode, 0, 0, 0);
        end;
    end;
    if jointDesc.bottomArm ~= nil then
        setRotation(jointDesc.bottomArm.rotationNode, jointDesc.bottomArm.rotX, jointDesc.bottomArm.rotY, jointDesc.bottomArm.rotZ);
        if jointDesc.bottomArm.translationNode ~= nil then
            setTranslation(jointDesc.bottomArm.translationNode, 0, 0, 0);
        end;
    end;
    if jointDesc.rotationNode ~= nil then
        setRotation(jointDesc.rotationNode, unpack(jointDesc.minRot));
    end;

    table.remove(self.attachedImplements, index);
    self.selectedImplement = math.min(self.selectedImplement, table.getn(self.attachedImplements));
    if self.selectedImplement ~= 0 then
        self.attachedImplements[self.selectedImplement].object:onSelect();
    end;

end;

function Vehicle4:detachImplementByObject(object)

    for i=1, table.getn(self.attachedImplements) do
        if self.attachedImplements[i].object == object then
            self:detachImplement(i);
            break;
        end;
    end;

end;

function Vehicle4:getImplementByObject(object)
    for i=1, table.getn(self.attachedImplements) do
        if self.attachedImplements[i].object == object then
            return self.attachedImplements[i];
        end;
    end;
    return nil;
end;

function Vehicle4:setSelectedImplement(selected)
    if self.selectedImplement ~= 0 then
        self.attachedImplements[self.selectedImplement].object:onDeselect();
    end;
    self.selectedImplement = selected;
    self.attachedImplements[selected].object:onSelect();
end;

function Vehicle4:playAttachSound()
    if self.attachSound ~= nil then
        setSamplePitch(self.attachSound, self.attachSoundPitchOffset);
        playSample(self.attachSound, 1, 1, 0);
    end;
end;

function Vehicle4:attachTrailer(trailer)
    if self.trailerAttacherJoint ~= nil then
        local constr = JointConstructor:new();
        self.attachedTrailer = trailer;
        self.attachedTrailer:onAttach(self);
        constr:setActors(self.rootNode, self.attachedTrailer.attachRootNode);
        constr:setJointTransforms(self.trailerAttacherJoint, self.attachedTrailer.attacherJoint);
        --constr:setBreakable(40, 40);
        constr:setRotationLimit(0, Utils.degToRad(-10), Utils.degToRad(10));
        constr:setRotationLimit(1, Utils.degToRad(-50), Utils.degToRad(50));
        constr:setRotationLimit(2, Utils.degToRad(-50), Utils.degToRad(50));
        self.trailerAttacherJointIndex = constr:finalize();
    end;
end;

function Vehicle4:setFuelFillLevel(newFillLevel)
    self.fuelFillLevel = math.max(math.min(newFillLevel, self.fuelCapacity), 0);
end;

function Vehicle4:setSpeedLevel(level)

    if level == 1 then
        --self.maxAcceleration =
    elseif level == 2 then
    elseif level == 3 then
    else
    end;

end;

function Vehicle4:handleAttachEvent()

    --if self.attachedTrailer ~= nil and g_currentMission.isTipTriggerInRange then
    --    self.attachedTrailer:toggleTipState();
    if g_currentMission.trailerInTipRange ~= nil then
        g_currentMission.trailerInTipRange:toggleTipState();
    elseif g_currentMission.attachableInMountRange ~= nil and self.attacherJoints[g_currentMission.attachableInMountRangeIndex].jointIndex == 0 then
        self:playAttachSound();
        self:attachImplement(g_currentMission.attachableInMountRange, g_currentMission.attachableInMountRangeIndex);
    elseif self:handleAttachTrailerEvent() then
        self:playAttachSound();
        -- nothing todo
    elseif self.attachedTrailer ~= nil then
        self:handleDetachTrailerEvent()
        self:playAttachSound();
    elseif self.selectedImplement ~= 0 then
        self:playAttachSound();
        self:detachImplement(self.selectedImplement);
    end;
end;

function Vehicle4:handleAttachTrailerEvent()
    if g_currentMission.trailerInMountRange ~= nil then
        if g_currentMission.trailerInMountRangeVehicle == self then
            if self.attachedTrailer == nil then
                self:attachTrailer(g_currentMission.trailerInMountRange);
				self.rotationMaxAttacherPart = true;
				self.translationMax = true;
                return true;
            end;
        elseif self.attachedTrailer ~= nil then
            return self.attachedTrailer:handleAttachTrailerEvent();
        end;
    end;
    return false;
end;

function Vehicle4:handleDetachTrailerEvent()

    if not self.attachedTrailer:handleDetachTrailerEvent() then
        if self.trailerAttacherJointIndex ~= 0 then
            removeJoint(self.trailerAttacherJointIndex);
            self.trailerAttacherJointIndex = 0;
			self.rotationMaxAttacherPart = false;
			self.translationMax = false;
            self.attachedTrailer:onDetach();
            self.attachedTrailer = nil;
        end;
    end;
end;

function Vehicle4:handleLowerImplementEvent()
    if self.selectedImplement ~= 0 then
        local implement = self.attachedImplements[self.selectedImplement];
        local jointDesc = self.attacherJoints[implement.jointDescIndex];
        jointDesc.moveDown = not jointDesc.moveDown;
    end;
end;

function Vehicle4:frontLoaderAttachableInRange()
    local nearestAttachable = nil;
    if self.isEntered ~= nil then
        local nearestDistance = 1.0;
        local jointDesc = self.frontLoaderJointDesc;
        local px, py, pz = getWorldTranslation(jointDesc.jointTransform);

        for i=1, table.getn(g_currentMission.attachables) do
            if not g_currentMission.attachables[i].isAttached then
                local vx, vy, vz = getWorldTranslation(g_currentMission.attachables[i].attacherJoint);
                local distance = Utils.vector3Length(px-vx, py-vy, pz-vz);
                if distance < nearestDistance then
                    nearestAttachable = g_currentMission.attachables[i];
                    nearestDistance = distance;
                end;
            end;
        end;
    end;
    return nearestAttachable;
end;